# -*- coding: utf-8 -*-

import json
import requests
from django.apps import apps
from django.conf import settings
from djangoplus.docs import utils
from django.utils import translation
from django.utils.translation import ugettext as _
from djangoplus.utils import get_metadata, get_register_parameters_details, get_parameters_details
from djangoplus.ui.components import forms


class Documentation(object):

    def __init__(self):
        from djangoplus.cache import CACHE
        from djangoplus.docs.usecase import Actor, UseCase
        from djangoplus.docs.diagrams import Workflow, ClassDiagram

        translation.activate(settings.LANGUAGE_CODE)

        self.description = None
        self.workflow = None
        self.actors = []
        self.usecases = []
        self.class_diagrams = []

        # load description
        for app_config in apps.get_app_configs():
            if app_config.label == settings.PROJECT_NAME:
                self.description = app_config.module.__doc__ and app_config.module.__doc__.strip() or None

        # load actors
        self.organization_model = CACHE['ORGANIZATION_MODEL']
        self.unit_model = CACHE['UNIT_MODEL']

        for model in CACHE['ROLE_MODELS']:
            name = CACHE['ROLE_MODELS'][model]['name']
            scope = CACHE['ROLE_MODELS'][model]['scope']
            description = utils.extract_documentation(model)
            self.actors.append(Actor(name=name, scope=scope, description=description))

        # load usecases
        self.workflow = Workflow()
        for task in self.workflow.tasks:
            if not task.startswith(_('Access')):
                usecase = UseCase(task)
                self.usecases.append(usecase)

        # load class diagrams
        for class_diagram_name, models in list(CACHE['CLASS_DIAGRAMS'].items()):
            class_diagram = ClassDiagram(class_diagram_name, models)
            self.class_diagrams.append(class_diagram)

    def as_dict(self):
        return dict(description=self.description, actors=[actor.as_dict() for actor in self.actors],
                    usecases=[usecase.as_dict() for usecase in self.usecases],
                    class_diagrams=[class_diagram.as_json() for class_diagram in self.class_diagrams],
                    organization_model=self.organization_model, unit_model=self.unit_model)

    def as_json(self):
        return json.dumps(self.as_dict())


class ApiDocumentation(object):

    def __init__(self):
        self.title = 'Index'
        self.index = []
        self.endpoints = []
        self.model = None

    def load_models(self):
        from djangoplus.cache import CACHE
        for model in CACHE['API_MODELS']:
            app_label = get_metadata(model, 'app_label')
            verbose_name_plural = get_metadata(model, 'verbose_name_plural')
            model_name = model.__name__.lower()
            url = '/docs/api/{}/{}/'.format(app_label, model_name)
            self.index.append((verbose_name_plural, url))

    def load_model(self, app_label, model_name, endpoint_name):
        from djangoplus.cache import CACHE
        model = apps.get_model(app_label, model_name)
        self.model = model
        self.title = get_metadata(model, 'verbose_name')

        name = 'add'
        url = '/api/{}/{}/'.format(app_label, model_name)
        doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
        self.add_endpoint(name, url, 'post', doc_url, [], get_register_parameters_details(model))

        name = 'edit'
        url = '/api/{}/{}/{{}}/'.format(app_label, model_name)
        doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
        self.add_endpoint(name, url, 'post', doc_url, [], get_register_parameters_details(model))

        name = 'delete'
        url = '/api/{}/{}/{{}}/'.format(app_label, model_name)
        doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
        self.add_endpoint(name, url, 'delete', doc_url, [], [])

        name = 'list'
        url = '/api/{}/{}/'.format(app_label, model_name)
        doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
        query_params = [
            dict(verbose_name='Page', name='page', type='Integer', required=False, help_text=None)
        ]
        self.add_endpoint(name, url, 'get', doc_url, query_params, [])

        for subset in CACHE['SUBSETS'][model]:
            name = subset['name']
            url = '/api/{}/{}/{}/'.format(app_label, model_name, name)
            doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
            query_params = [
                dict(verbose_name='Fields', name='fields', type='List[String]', required=False, help_text='Fields to be displayed')
            ]
            self.add_endpoint(name, url, 'get', doc_url, query_params, [])

        for method in CACHE['MANAGER_METHODS'].get(model, []):
            name = method['function']
            url = '/api/{}/{}/{}/'.format(app_label, model_name, name)
            doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
            self.add_endpoint(name, url, 'get', doc_url, [], [])

        name = 'get'
        url = '/api/{}/{}/{{}}/'.format(app_label, model_name)
        doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
        query_params = [
            dict(verbose_name='Fields', name='fields', type='List[String]', required=False, help_text='Fields to be displayed')
        ]
        self.add_endpoint(name, url, 'get', doc_url, query_params, [])

        for method in CACHE['INSTANCE_METHODS'].get(model, []):
            name = method['function']
            url = '/api/{}/{}/{{}}/{}/'.format(app_label, model_name, name)
            doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, name)
            self.add_endpoint(name, url, 'get', doc_url, [], [])

        for group in CACHE['INSTANCE_ACTIONS'][model]:
            for func_name in CACHE['INSTANCE_ACTIONS'][model][group]:
                action = CACHE['INSTANCE_ACTIONS'][model][group][func_name]
                if action['source'] == 'model':
                    url = '/api/{}/{}/{{}}/{}/'.format(app_label, model_name, func_name)
                    doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, func_name)
                    func = getattr(model(), func_name)
                    input_params = get_parameters_details(model, func, action['input'])
                    self.add_endpoint(func_name, url, 'post', doc_url, [], input_params)

        for group in CACHE['QUERYSET_ACTIONS'][model]:
            for func_name in CACHE['QUERYSET_ACTIONS'][model][group]:
                action = CACHE['QUERYSET_ACTIONS'][model][group][func_name]
                if action['source'] == 'model':
                    url = '/api/{}/{}/{}/'.format(app_label, model_name, func_name)
                    doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, func_name)
                    func = getattr(model.objects.all(), func_name)
                    query_params = [
                        dict(verbose_name='Ids', name='ids', type='Integer', required=True,
                             help_text='Integer or list of integers separanted by ",". Ex: 1 or 1,2')
                    ]
                    input_params = get_parameters_details(model, func, action['input'])
                    self.add_endpoint(func_name, url, 'post', doc_url, query_params, input_params)

        for group in CACHE['CLASS_ACTIONS'][model]:
            for func_name in CACHE['CLASS_ACTIONS'][model][group]:
                action = CACHE['CLASS_ACTIONS'][model][group][func_name]
                if action['source'] == 'model':
                    url = '/api/{}/{}/{}/'.format(app_label, model_name, func_name)
                    doc_url = '/docs/api/{}/{}/{}/'.format(app_label, model_name, func_name)
                    func = getattr(model.objects.all(), func_name)
                    input_params = get_parameters_details(model, func, action['input'])
                    self.add_endpoint(func_name, url, 'post', doc_url, [], input_params)

        for endpoint in self.endpoints:
            if endpoint['name'] == endpoint_name:
                return endpoint
        return None

    def add_endpoint(self, name, url, method, doc_url, query_params, input_params):
        endpoint = dict(
            name=name, url=url, method=method, doc_url=doc_url,
            vars=vars, query_params=query_params, input_params=input_params
        )
        self.endpoints.append(endpoint)

    def form_cls(self, endpoint, token, host):

        class ApiForm(forms.Form):

            class Meta:
                title = ''
                cancel_button = False
                submit_label = 'Execute'

            def __init__(self, *args, **kwargs):
                super(ApiForm, self).__init__(*args, **kwargs)
                self.query_params_fields = []
                self.input_params_fields = []
                self.fields['api'] = forms.CharField(widget=forms.widgets.HiddenInput(), required=False)
                if '{}' in endpoint['url']:
                    self.fields['pk'] = forms.CharField(label='ID', required=True)
                for param in endpoint['query_params']:
                    self.fields[param['name']] = forms.CharField(label=param['verbose_name'], required=param['required'], help_text=param['help_text'])
                    self.query_params_fields.append(param['name'])
                for param in endpoint['input_params']:
                    self.fields[param['name']] = forms.CharField(label=param['verbose_name'], required=param['required'], help_text=param['help_text'])
                    self.input_params_fields.append(param['name'])

                self.fieldsets = []
                if 'pk' in self.fields:
                    self.fieldsets.append((u'Identifier', {'fields': ('pk',)}))
                if self.query_params_fields:
                    self.fieldsets.append((u'Query Params', {'fields': self.query_params_fields}))
                if self.input_params_fields:
                    self.fieldsets.append((u'Input Params', {'fields': self.input_params_fields}))

            def process(self):
                if 'pk' in self.fields:
                    url = endpoint['url'].format(self.cleaned_data['pk'])
                else:
                    url = endpoint['url']
                data = dict()
                input_params = []
                for field_name in self.input_params_fields:
                    data[field_name] = self.cleaned_data[field_name]
                    input_params.append('{}={}'.format(field_name, self.cleaned_data[field_name]))
                query_params = []
                for field_name in self.query_params_fields:
                    query_params.append('{}={}'.format(field_name, self.cleaned_data[field_name]))
                if query_params:
                    url = '{}?{}'.format(url, '&'.join(query_params))
                url = 'http://{}{}'.format(host, url)
                headers = {'Authorization': 'Token {}'.format(token)}
                if endpoint['method'] == 'post':
                    call_func = requests.post
                elif endpoint['method'] == 'delete':
                    call_func = requests.delete
                elif endpoint['method'] == 'put':
                    call_func = requests.put
                else:
                    call_func = requests.get
                response = call_func(url, data=self.cleaned_data, headers=headers)
                extra = input_params and '-d "{}"'.format('&'.join(input_params)) or ''
                cmd = 'curl -X {} -H "Authorization: Token {}" {} {}'.format(endpoint['method'].upper(), token, extra, url)
                return cmd, response.content.decode()

        return ApiForm



# -*- coding: utf-8 -*-

import datetime
from collections import OrderedDict
from djangoplus.ui.components import Component
from djangoplus.utils import normalyze
from djangoplus.utils.metadata import get_metadata
from djangoplus.utils.serialization import dumps_qs_query
from djangoplus.utils.permissions import has_add_permission
from djangoplus.utils.dateutils import add_days, DAY_INITIALS


class Calendar(Component):
    def __init__(self, request, title, url=None):
        super(Calendar, self).__init__(title, request)
        self.title = title
        self.items = []
        self.lazy = False
        self.initial_date = None
        self.url = url

        self.set_initial_date(datetime.date.today())

    def add(self, description, start, end=None, url=None, allday=True):
        item = dict(description=description, start=start, end=end, allday=allday)
        self.items.append(item)
    
    def set_initial_date(self, initial_date):
        self.initial_date = initial_date
        session_initial_date = self.request.session.get('calendar_initial_date')
        if session_initial_date:
            self.initial_date = datetime.datetime.strptime(session_initial_date, '%Y-%m-%d')


class AnnualCalendar(Component):

    formatter_name = 'annual_calendar'

    def __init__(self, data, title=None, compact=False):
        super().__init__()
        self.title = title
        self.compact = compact
        self.calendars = []
        self.items = []
        self.caption = OrderedDict()

        for values in data or []:
            self.add(*values)

    def add(self, description, start, end=None, color='#FFF', detail=None):
        item = dict(description=description, start=start, end=end, color=color, detail=detail)
        self.items.append(item)
        if color not in self.caption:
            self.caption[color] = description

    def __str__(self):
        today = datetime.date.today()
        header = DAY_INITIALS[0:-1]
        header.insert(0, DAY_INITIALS[-1])
        for i in range(1, 13):
            first_day = datetime.date(today.year, i, 1)
            calendar = [header]
            details = {}
            line = []
            date = first_day
            month = date.month
            weekday = date.weekday()
            weekday = weekday is not 6 and weekday + 1 or 0
            for day in range(0, weekday):
                line.append(('', '#FFF', '#777'))
            while date.month == month:
                weekday = date.weekday()
                weekday = weekday is not 6 and weekday + 1 or 0
                bgcolor = weekday in (0, 6) and '#EEE' or '#FFF'
                fcolor = '#777'
                for item in self.items:
                    add_detail = False
                    if item['start'] and item['end']:
                        if item['start'] <= date <= item['end']:
                            bgcolor = item['color']
                            fcolor = '#FFF'
                            add_detail = True
                    else:
                        if date == item['start']:
                            bgcolor = item['color']
                            fcolor = '#FFF'
                            add_detail = True
                    if item['detail'] and add_detail:
                        if item['detail'] not in details:
                            details[item['detail']] = []
                        details[item['detail']].append(date.day)
                line.append((date.day, bgcolor, fcolor))
                if weekday == 6:
                    calendar.append(line)
                    line = []
                date = add_days(date, 1)
            while len(line) < 7:
                line.append(('', '#FFF', '#777'))
            if len(calendar) < 6:
                calendar.append(line)
            visible = i in (today.month-1, today.month, today.month+1)
            self.calendars.append((normalyze(first_day.strftime('%B')), calendar, details, visible))
        return super().__str__()


class AnnualCompactCalendar(AnnualCalendar):
    template_name = 'annualcalendar.html'
    formatter_name = 'annual_compact_calendar'

    def __init__(self, items, title=None):
        super().__init__(items, title=title, compact=True)


class ModelCalendar(Calendar):
    def __init__(self, request, title, editable=False, display_time=True, url=None):
        super(ModelCalendar, self).__init__(request, title, url=url)
        self.links = []
        self.lazy = True
        self.models = []
        self.editable = editable
        self.display_time = display_time

    def add(self, queryset, start_field, end_field=None, color='#ccc', action_names=None, as_initial_date=False):
        if action_names is None:
            action_names = []
        item = dict(
            queryset=dumps_qs_query(queryset), start_field=start_field,
            end_field=end_field, color=color, action_names=','.join(action_names)
        )
        self.items.append(item)
        if as_initial_date:
            qs_initial_date = queryset.order_by(start_field).values_list(start_field, flat=True)
            if qs_initial_date.exists():
                self.set_initial_date(qs_initial_date[0])
        if queryset.model not in self.models and has_add_permission(self.request, queryset.model):
            label = get_metadata(queryset.model, 'verbose_name')
            app_label = get_metadata(queryset.model, 'app_label')
            model_name = queryset.model.__name__.lower()
            url = '/add/{}/{}/?{}='.format(app_label, model_name, start_field)
            link = dict(label=label, url=url)
            self.links.append(link)
            self.models.append(queryset.model)



# -*- coding: utf-8 -*-
from django.db.models.deletion import Collector
from django.core import serializers
from django.core.management.base import BaseCommand
import pdb

def dump(qs):
    l = []
    c = []
    collector = Collector('default')
    def dump_related_objects(instance):
        if instance.__class__.__name__ not in ['Log', 'RegistroDiferenca'] and instance not in l:
            l.append(instance)
            if instance.__class__.__name__ not in c:
                c.append(instance.__class__.__name__)

            related_fields = [
                f for f in instance.__class__._meta.get_fields(include_hidden=True)
                if (f.one_to_many or f.one_to_one)
                and f.auto_created and not f.concrete
            ]
            
            qs = instance.__class__.objects.filter(pk=instance.pk)
            for related_field in related_fields:
                objs = collector.related_objects(related_field, qs)
                for obj in objs:
                    if obj not in l:
                        dump_related_objects(obj)
    for instance in qs:
        print('Loading', instance.pk, instance)
        dump_related_objects(instance)
    return c, l

class Command(BaseCommand):
    def handle(self, *args, **options):
        queryset = None
        print('\nPlease, instanciate the queryset to be saved and then press the key "c".\nThe models must be previously imported.\n\tEx: from djangoplus.admin.models import User\n\tqueryset = User.objects.filter(pk=1)\n\n')
        pdb.set_trace()
        c, l = dump(queryset)
        print('A "dump.json" was saved in the current directory with %s objects. In order to restore then, type:\n\n\tpython manage.py restore %s < dump.json\n\n' % (len(l) , ' '.join(c)))
        f = open('dump.json', 'w')
        f.write(serializers.serialize("json", l))
        f.close()



# -*- coding: utf-8 -*-
import json
import zlib
import base64
import types
import _pickle as cpickle
from django.apps import apps
from django.core import signing
from django.db.models import QuerySet, Model
from django.utils.text import slugify

from djangoplus.utils import get_fieldsets, iterable
from djangoplus.utils.formatter import format_value
from djangoplus.utils.metadata import get_metadata, getattr2


def dumps_qs_query(qs):
    serialized_str = base64.b64encode(zlib.compress(cpickle.dumps(qs.query))).decode()
    payload = {
        'model_label': getattr(qs.model, '_meta').label,
        'query': serialized_str,
    }
    return signing.dumps(payload)


def loads_qs_query(data):
    payload = signing.loads(data)
    model = apps.get_model(payload['model_label'])
    queryset = model.objects.none()
    queryset.query = cpickle.loads(zlib.decompress(base64.b64decode(payload['query'])))
    return queryset


def serialize(attr_value, recursive=False):
    if isinstance(attr_value, types.MethodType) or isinstance(attr_value, types.FunctionType):
        attr_value = attr_value()
    if isinstance(attr_value, QuerySet) or hasattr(attr_value, 'all'):
        serialized_value = []
        for obj in attr_value.all():
            serialized_value.append(serialize(obj))
    elif isinstance(attr_value, Model):
        obj = attr_value
        obj_dict = dict(id=obj.pk)
        if recursive:
            obj_dict['text'] = str(obj)
        else:
            for attr_name in get_metadata(type(attr_value), 'list_display'):
                obj_dict[attr_name] = serialize(getattr(obj, attr_name), recursive=True)
        serialized_value = obj_dict
    elif type(attr_value).__name__ == 'QueryStatistics':
        query_statistics_table = attr_value.as_table()
        serialized_value = dict(
            header=query_statistics_table.header,
            rows=query_statistics_table.rows,
            footer=query_statistics_table.footer
        )
    else:
        serialized_value = format_value(attr_value, False)
    return serialized_value


def json_serialize(value, query_params, **extras):
    if isinstance(value, Model):
        serialized_value = dict(id=value.id)
        model = type(value)
        only = query_params.get('fields', [])
        fieldsets = get_metadata(model, 'fieldsets', get_fieldsets(model))
        for title, data in fieldsets:
            title = slugify(title.split('::')[-1]).replace('-', '_')
            if not only or title in only:
                serialized_value[title] = dict()
                fields = data.get('fields', [])
                for field_or_truple in fields:
                    for attr_name in iterable(field_or_truple):
                        if len(fields) == 1:
                            serialized_value[title] = serialize(getattr(value, attr_name), True)
                        else:
                            serialized_value[title][attr_name] = serialize(getattr(value, attr_name), True)
                relations = data.get('relations', [])
                for attr_name in relations:
                    if len(relations) == 1 and not fields:
                        serialized_value[title] = serialize(getattr(value, attr_name))
                    else:
                        serialized_value[title][attr_name] = serialize(getattr(value, attr_name))
    else:
        serialized_value = serialize(value)

    output_data = dict(result=serialized_value)
    for key, value in extras.items():
        output_data[key] = value
    try:
        return json.dumps(output_data, indent=4, sort_keys=True, ensure_ascii=False)
    except TypeError as e:
        return json.dumps(dict(error=str(e)))



# -*- coding: utf-8 -*-

from django.conf import settings
from djangoplus.docs import utils
from djangoplus.cache import CACHE
from django.http import HttpRequest
from django.utils import translation
from djangoplus.admin.models import User
from django.utils.translation import ugettext as _
from django.template.defaultfilters import slugify
from djangoplus.db.models import fields as model_fields
from djangoplus.ui.components.forms import factory, fields as form_fields
from djangoplus.utils import get_fieldsets
from djangoplus.utils.metadata import get_metadata, find_action, find_model_by_verbose_name,\
    find_model_by_add_label, get_field, find_model_by_verbose_name_plural, find_subset_by_title, get_parameters_names


class Actor(object):
    def __init__(self, name=None, description=None, scope=None):
        self.name = name
        self.description = description
        self.scope = scope

    def __str__(self):
        return self.name

    def as_dict(self):
        return dict(name=self.name, description=self.description, scope=self.scope)


class UseCase(object):
    
    ACTOR = None

    """ This class has the aim of generating interactions and testcases based on usecases.
    The code generation is based on strings that holds the usercase name.
    The usecases must follow the sintaxe bellow:
        Login as <group name>
        Register <class verbose name>
        List <class plural verbose name>
        List <class plural verbose name>: <subset name>
        Add <relation name> to <class verbose name>
        <action name> in <class verbose name>
        <action name>
    Code generation is also compatible with Porguese and may also follow the sintaxe bellow
        Acessar como <nome do grupo>
        Cadastrar <nome amigável da classe>
        Listar <nome amigável da classe no plural>
        Listar <nome amigável da classe no plural>: <nome do subconjunto>
        Adicionar <nome da relação> em <nome amigável da classe>
        <nome da ação> em <nome amigável da classe>
        <nome da ação>
    """

    def __init__(self, name):

        translation.activate(settings.LANGUAGE_CODE)

        self._interactions = []
        self._func_signature = None
        self._test_function_code = []

        self.name = name
        self.description = None
        self.actors = []
        self.business_rules = []
        self.pre_conditions = []
        self.post_condition = None

        self._request = None

        if name.startswith(_('Access')):
            self._login(name)
        elif name.startswith(_('List')):
            self._list(name)
        elif name.startswith(_('Register')) and ' como ' in name:
            self._signup(name)
        elif name.startswith(_('Register')) and _('in') not in name:
            self._register(name)
        elif name.startswith(_('Add ')):
            self._add(name)
        else:
            self._execute(name)

    def _mocked_request(self):
        if not self._request:
            self._request = HttpRequest()
            self._request.user = User(pk=0)
        return self._request

    def _debug(self):
        print('\n'.join(self._interactions))
        print()
        print('\n'.join(self._test_function_code))

    def _login(self, action):
        verbose_name = action.split(_('as'))[-1].strip()
        CACHE['LAST_AUTHENTICATED_ROLE'] = verbose_name
        model = find_model_by_verbose_name(verbose_name)
        if model:
            role_username = get_metadata(model, 'role_username')
            if role_username:
                field = get_field(model, role_username)
                CACHE['LAST_AUTHENTICATED_USERNAME'] = field.example or None
        interaction = _('The user access the system as')
        self._interactions.append('{} {}'.format(interaction, verbose_name))
        self._interactions.append(_('The system displays the main page'))

    def _find(self, model, registering=False):
        click_str = []
        click_str_unicode = []

        verbose_name_plural = get_metadata(model, 'verbose_name_plural')
        list_shortcut = get_metadata(model, 'list_shortcut', [], iterable=True)
        list_menu = get_metadata(model, 'list_menu', [], iterable=True)
        dashboard = get_metadata(model, 'dashboard')
        menu = get_metadata(model, 'menu')

        # list_required_role = [role_name for role_name in (list_shortcut + list_menu) if role_name is not True]
        # if list_required_role and CACHE['LAST_AUTHENTICATED_ROLE'] and CACHE['LAST_AUTHENTICATED_ROLE'] not in list_required_role:
        #    self._login('{} {} {}'.format(_('Access'), _('as'), list_required_role[0]))

        if dashboard and not registering:
            panel_title = verbose_name_plural
            interaction = _('The user looks at the painel')
            self._interactions.append('{} "{}"'.format(interaction, panel_title))
            self._test_function_code.append("        self.look_at_panel('{}')".format(panel_title))
            return True
        elif not CACHE['LAST_AUTHENTICATED_ROLE'] or True in list_shortcut or CACHE['LAST_AUTHENTICATED_ROLE'] in list_shortcut:

            left = _('The user clicks on the shortcut card')
            right = _('in the dashboard at main page')
            interaction = '{} "{}" {}'.format(left, verbose_name_plural, right)
            self._interactions.append(interaction)

            self._interactions.append(_('The system displays the listing page'))

            self._test_function_code.append('        self.click_link(u\'{}\')'.format(verbose_name_plural))
            return True
        elif menu:

            if type(menu) == tuple:
                menu = menu[0]
            for menu_item in menu.split('::'):
                click_str.append('"{}"'.format(menu_item))
                click_str_unicode.append("'{}'".format(menu_item))

            interaction = _('The user access the menu')
            self._interactions.append('{} {}'.format(interaction, ', '.join(click_str)))

            self._test_function_code.append('        self.click_menu({})'.format(', '.join(click_str_unicode)))
            return True
        return False

    def _view(self, model, recursive=False):
        accessible = False

        verbose_name = get_metadata(model, 'verbose_name')
        list_shortcut = get_metadata(model, 'list_shortcut', [])
        list_menu = get_metadata(model, 'menu')

        # if the model can be accessed by the menu or shortcut
        if (list_shortcut or list_menu) and self._find(model):
            accessible = True
            self._interactions.append(_('The user locates the record and clicks the visualization icon'))
            self._interactions.append(_('The system displays the visualization page'))
            self._test_function_code.append("        self.click_icon('{}')".format('Visualizar'))  # _('Visualize')
        else:
            # the model can be accessed only by a parent model
            for parent_model in CACHE['COMPOSITION_RELATIONS']:
                if model in CACHE['COMPOSITION_RELATIONS'][parent_model]:
                    self._view(parent_model, True)
                    panel_title = None
                    if hasattr(parent_model, 'fieldsets'):
                        for fieldset in parent_model.fieldsets:
                            for item in fieldset[1].get('relations', ()) + fieldset[1].get('inlines', ()):
                                relation = getattr(parent_model, item.split(':')[0].split('__')[0])
                                if hasattr(relation, 'field') and relation.field.remote_field.related_model == model:
                                    panel_title = fieldset[0]
                                    break
                    if panel_title:
                        accessible = True
                        if '::' in panel_title:
                            tab_name, panel_title = panel_title.split('::')
                            self._test_function_code.append("        self.click_tab('{}')".format(tab_name))
                            interaction = _('The user clicks the tab')
                            self._interactions.append('{} "{}"'.format(interaction, tab_name))
                        if panel_title:
                            interaction = _('The user looks at the painel')
                            self._interactions.append('{} "{}"'.format(interaction, panel_title))
                            self._test_function_code.append("        self.look_at_panel('{}')".format(panel_title))

                    if recursive:
                        self._interactions.append(_('The user locates the record and clicks the visualization icon'))
                        self._interactions.append(_('The system displays the visualization page'))
                        self._test_function_code.append("        self.click_icon('{}')".format('Visualizar'))  # _('Visualize')

        if not accessible:
            raise ValueError(
                'There is no way to access the model "{}" from "", please do one of the following things:\n '
                'i) Add the "list_menu" meta-attribute to the model\n '
                'ii) Add the "list_shortcut" meta-attribute to the model\n '
                'iii) Add the "composition" field attribute to one of foreignkey fields of the model if '
                'it exists and the the relation on the foreignkey model'.format(verbose_name)
            )

    def _list(self, action):
        verbose_name_plural = action.replace(_('List'), '').strip()

        if ':' in action:
            # it refers to a subset
            verbose_name_plural, subset_name = verbose_name_plural.split(':')
            verbose_name_plural = verbose_name_plural.strip()
            model = find_model_by_verbose_name_plural(verbose_name_plural)
            subset_name = subset_name.strip()
            subset = find_subset_by_title(subset_name, model)

            # set the attributes of the usecase
            self.description = (subset['function'].__doc__ or '').strip()
            for can_view in subset['can_view']:
                self.actors.append(can_view)
            if not self.actors:
                self.actors.append('Superusuário')

            # register the interactions and testing code
            self._find(model)

            interaction = _('The user clicks the pill')
            self._interactions.append('{} "{}"'.format(interaction, subset_name))

            self._test_function_code.append("        self.click_link('{}')".format(subset_name))

        else:
            model = find_model_by_verbose_name_plural(verbose_name_plural)

            # set the attributes of the usecase
            self.description = 'List {} previouly registered in the system'.format(verbose_name_plural.lower())
            self.post_condition = _('The system displays the registered records')
            for meta_data in ('can_admin', 'can_admin_by_role', 'can_admin_by_unit', 'can_admin_by_organization',
                              'can_view', 'can_view_by_role', 'can_view_by_unit', 'can_view_by_organization'):
                for actor in get_metadata(model, meta_data, iterable=True):
                    if actor:
                        self.actors.append(actor)
            if not self.actors:
                self.actors.append('Superusuário')

            # register the interactions and testing code
            self._find(model)

        # register the interactions
        search_fields = utils.get_search_fields(model)
        if search_fields:
            interaction = _('The user optionally performs a search by')
            self._interactions.append('{} {}'.format(interaction, search_fields))

        list_filter = utils.get_list_filter(model)
        if list_filter:
            interaction = _('The user optionally filter the records by')
            self._interactions.append('{} {}'.format(interaction, list_filter))

        left = _('The system displays')
        right = _('of previous registered records')
        interaction = '{} {} {}'.format(left, utils.get_list_display(model), right)
        self._interactions.append(interaction)

    def _signup(self, action):
        model_name = action.split('como')[1].strip()
        model = find_model_by_verbose_name(model_name)

        func_name = slugify(action).replace('-', '_')
        func_decorator = '@testcase(\'{}\', None)'.format(self.name)
        self._func_signature = '{}(self)'.format(func_name)

        self._test_function_code.append('    {}'.format(func_decorator))
        self._test_function_code.append('    def {}:'.format(self._func_signature))

        interaction = _('The user access login page')
        self._interactions.append(interaction)
        self._test_function_code.append("        self.open('/admin/login')")

        interaction = _('The user clicks the link')
        self._interactions.append('{} "{}"'.format(interaction, _('Sign-up')))
        self._test_function_code.append("        self.click_button('{}')".format(_('Sign-up')))

        self._interactions.append(_('The system displays a popup window'))
        self._test_function_code.append("        self.look_at_popup_window()")

        form = factory.get_register_form(self._mocked_request(), model())
        self._fill_out(form)

        interaction = _('The user clicks the button')
        self._interactions.append('{} "{}"'.format(interaction, _('Register')))
        self._test_function_code.append("        self.click_button('{}')".format(_('Register')))

    def _register(self, action):
        model = find_model_by_add_label(action)
        if model:
            verbose_name = get_metadata(model, 'verbose_name')
            button_label = get_metadata(model, 'add_label')
            save_button_label = get_metadata(model, 'add_label', _('Save'))
            func_name = slugify(button_label).replace('-', '_')
        else:
            verbose_name = action.replace(_('Register'), '').strip()
            model = find_model_by_verbose_name(verbose_name)
            button_label = _('Register')
            save_button_label = _('Save')
            func_name = 'cadastrar_{}'.format(model.__name__.lower())

        # set the attributes of the usecase
        self.name = action
        self.description = '{} {} {}'.format(_('Add new records of'),  verbose_name.lower(), _('in the system'))
        self.business_rules = utils.extract_exception_messages(model.save)
        self.post_condition = _('The record will be successfully registered in the system')
        required_data = []
        for field in get_metadata(model, 'get_fields'):
            if isinstance(field, model_fields.ForeignKey):
                if not isinstance(field, model_fields.OneToOneField):
                    if not isinstance(field, model_fields.OneToManyField):
                        required_data.append(field.verbose_name.lower())
        if required_data:
            pre_condition = _('The following information must have been previouly registered in the system: ')
            self.pre_conditions.append('{} {}'.format(pre_condition, ', '.join(required_data)))

        for meta_data in ('can_admin', 'can_admin_by_role', 'can_admin_by_unit', 'can_admin_by_organization', 'can_add',
                          'can_add_by_role', 'can_add_by_unit', 'can_add_by_organization'):
            for actor in get_metadata(model, meta_data, iterable=True):
                if actor:
                    self.actors.append(actor)
        if not self.actors:
            self.actors.append(_('Superuser'))

        # register the interactions and testing code'
        username_attr = ''
        if CACHE['LAST_AUTHENTICATED_USERNAME']:
            username_attr = ", '{}'".format(CACHE['LAST_AUTHENTICATED_USERNAME'])
        func_decorator = '@testcase(\'{}\'{})'.format(self.name, username_attr)
        self._func_signature = '{}(self)'.format(func_name)

        self._test_function_code.append('    {}'.format(func_decorator))
        self._test_function_code.append('    def {}:'.format(self._func_signature))
        self._find(model, registering=True)
        self._test_function_code.append("        self.click_button('{}')".format(button_label))

        a = _('The user clicks the button')
        b = _('on the right-side of the action bar')
        self._interactions.append('{} "{}" {}'.format(a, button_label, b))

        form = factory.get_register_form(self._mocked_request(), model())
        self._fill_out(form)

        interaction = _('The user clicks the button')
        self._interactions.append('{} "{}"'.format(interaction, save_button_label))
        self._test_function_code.append("        self.click_button('{}')".format(save_button_label))
        self._test_function_code.append("        self.click_icon('{}')".format('Principal'))

    def _add(self, action):
        model = None

        if action.startswith(_('Add ')):
            # not add_label was defined for the related model
            tokens = action.replace(_('Add '), '').split(_(' in '))
            verbose_name = tokens[0].strip()
            if len(tokens) > 1:
                relation_verbose_name = tokens[1].strip()
                model = find_model_by_verbose_name(relation_verbose_name)
            related_model = find_model_by_verbose_name(verbose_name)
        else:
            # an add_label was defined for the related model
            tokens = action.split(_(' in '))
            add_label = tokens[0].strip()
            if len(tokens) > 1:
                verbose_name = tokens[1].strip()
                model = find_model_by_verbose_name(verbose_name)
            related_model = find_model_by_add_label(add_label)

        # check if there is a fieldset was defined with the relation
        fieldsets = hasattr(model, 'fieldsets') and model.fieldsets or get_fieldsets(model)
        relation_name = None
        inlines = []
        if hasattr(model, 'fieldsets'):
            for fieldset in fieldsets:
                if 'relations' in fieldset[1]:
                    for item in fieldset[1]['relations']:
                        tmp = getattr(model, item.split(':')[0].split('__')[0])
                        if hasattr(tmp, 'field') and tmp.field.remote_field.model == model:
                            relation_name = item
                if 'inlines' in fieldset[1]:
                    for item in fieldset[1]['inlines']:
                        inlines.append(item)
                        tmp = getattr(model, item)
                        if hasattr(tmp, 'field') and tmp.field.remote_field.model == model:
                            relation_name = item

        # if the relation was defined in a fieldset
        if relation_name:
            add_inline = relation_name in inlines
            add_label = get_metadata(related_model, 'add_label')
            button_label = add_label or 'Adicionar'
            button_label = get_metadata(related_model, 'add_label', button_label)
            username_attr = ''
            if CACHE['LAST_AUTHENTICATED_USERNAME']:
                username_attr = ", '{}'".format(CACHE['LAST_AUTHENTICATED_USERNAME'])
            func_decorator = '@testcase(\'{}\'{})'.format(self.name, username_attr)
            self._func_signature = '{}_{}_{}_{}'.format(_('add'), related_model.__name__.lower(), _('in'), model.__name__.lower())

            self._test_function_code.append('    {}'.format(func_decorator))
            self._test_function_code.append('    def {}(self):'.format(self._func_signature))

            self._view(related_model)

            # the form is not in the visualization page and it must be opened
            if not add_inline:
                add_button_label = 'Adicionar {}'.format(get_metadata(related_model, 'verbose_name'))
                add_button_label = get_metadata(related_model, 'add_label', add_button_label)

                interaction = _('The user clicks the button')
                self._interactions.append('{} "{}"'.format(interaction, add_button_label))
                self._interactions.append(_('The system displays a popup window'))

                self._test_function_code.append("        self.click_button('{}')".format(add_button_label))
                self._test_function_code.append("        self.look_at_popup_window()")

            form = factory.get_many_to_one_form(self._mocked_request(), model(), relation_name, related_model())
            self._fill_out(form)

            save_button_label = get_metadata(related_model, 'add_label', _('Save'))
            interaction = _('The user clicks the button')
            self._interactions.append('{} "{}"'.format(interaction, save_button_label))
            self._test_function_code.append("        self.click_button('{}')".format(save_button_label))
            self._test_function_code.append("        self.click_icon('{}')".format('Principal'))
        else:
            raise ValueError('Please add the {}\'s relation in the fieldsets of model {}'.format(related_model.__name__, model.__name__))

    def _fill_out(self, form, inline=None):

        if not inline:
            self._interactions.append(_('The system displays the input form'))

        form.contextualize()
        form.configure()

        for fieldset in form.configured_fieldsets:
            if inline or fieldset['title']:
                if fieldset['tuples'] and fieldset['tuples'][0]:
                    interaction = _('The user looks at fieldset')
                    self._interactions.append('{} "{}"'.format(interaction, inline or fieldset['title']))

            for fields in fieldset['tuples']:
                if not type(fields) in (list, tuple):
                    fields = fields,
                field_names = [field['name'] for field in fields]

                for field_name in field_names:
                    if field_name in form.base_fields:
                        form_field = form.base_fields[field_name]

                        # it is necessary to skip "organization" and "unit" fields if the actor's scope does not match
                        if self.actors and hasattr(form_field, 'queryset'):
                            from djangoplus.admin.models import Organization, Unit
                            scopes = []
                            if issubclass(form_field.queryset.model, Organization):
                                scopes = ['systemic']
                            if issubclass(form_field.queryset.model, Unit):
                                scopes = ['systemic', 'organization']

                            if scopes:
                                ignore_field = False
                                for role_model in CACHE['ROLE_MODELS']:
                                    if CACHE['ROLE_MODELS'][role_model]['name'] == self.actors[0]:
                                        if CACHE['ROLE_MODELS'][role_model]['scope'] not in scopes:
                                            ignore_field = True
                                            break
                                if ignore_field:
                                    continue
                            # the field is related to the class associated to the user's role
                            if get_metadata(form_field.queryset.model, 'verbose_name') == CACHE['LAST_AUTHENTICATED_ROLE']:
                                continue

                        # get the test value in case of model form and if the sample value was defined in the model
                        value = ''
                        meta_class = getattr(form, '_meta', None)
                        if meta_class:
                            if hasattr(meta_class.model, field_name):
                                model_field = get_field(meta_class.model, field_name)
                                if hasattr(model_field, 'example') and model_field.example:
                                    value = callable(model_field.example) and model_field.example() or model_field.example

                        # define the kind of user interaction
                        if hasattr(form_field, 'choices') and form_field.choices:
                            self._test_function_code.append("        self.choose('{}', '{}')".format(form_field.label, value))
                            interaction = _('The user chooses')
                            self._interactions.append('{} "{}"'.format(interaction, form_field.label))
                        else:
                            if not isinstance(form_field, form_fields.BooleanField) and not isinstance(form_field, form_fields.NullBooleanField):
                                self._test_function_code.append("        self.enter('{}', '{}')".format(form_field.label, value))
                                interaction = _('The user enters')
                                self._interactions.append('{} "{}"'.format(interaction, form_field.label))

            # add the interactions recursively if the form inner forms
            for field, form, required, save in fieldset.get('one_to_one', ()):
                self._fill_out(form, inline=field.label)
            for name, field, one_to_many_forms, one_to_many_count in fieldset.get('one_to_many', ()):
                for form in one_to_many_forms[0:1]:
                    self._fill_out(form, inline=field.label)

    def _execute(self, action):
        tokens = action.split(_(' in '))
        if find_model_by_add_label(tokens[0].strip()):
            if len(tokens) == 1:
                self._register(action)
            else:
                self._add(action)
        else:
            if len(tokens) == 1:
                self._execute_view(action)
            else:
                self._execute_action(action)

    def _execute_action(self, action):
        tokens = action.split(_(' in '))
        action_name = tokens[0].strip()
        verbose_name = tokens[1].strip()
        model = find_model_by_verbose_name(verbose_name)
        action_dict = find_action(model, action_name)
        func = action_dict['function']
        username_attr = ''
        if CACHE['LAST_AUTHENTICATED_USERNAME']:
            username_attr = ", '{}'".format(CACHE['LAST_AUTHENTICATED_USERNAME'])
        func_decorator = '@testcase(\'{}\'{})'.format(self.name, username_attr)
        self._func_signature = '{}_em_{}(self)'.format(func.__name__.lower(), model.__name__.lower())

        self._test_function_code.append('    {}'.format(func_decorator))
        self._test_function_code.append('    def {}:'.format(self._func_signature))
        self._view(model, True)
        if hasattr(func, '_action'):
            button_label = func._action['verbose_name']
            params = get_parameters_names(func)
            if params:
                interaction = _('The user clicks the button')
                self._interactions.append('{} "{}"'.format(interaction, button_label))
                self._interactions.append(_('The system displays a popup window'))

                self._test_function_code.append("        self.click_button('{}')".format(button_label))
                self._test_function_code.append("        self.look_at_popup_window()")
                obj = model()
                obj.pk = 0
                form = factory.get_action_form(self._mocked_request(), obj, func._action)
                self._fill_out(form)

            interaction = _('The user clicks the button')
            self._interactions.append('{} "{}"'.format(interaction, button_label))
            self._test_function_code.append("        self.click_button('{}')".format(button_label))
            self._test_function_code.append("        self.click_icon('{}')".format('Principal'))

            description = func.__doc__ and func.__doc__.strip() or ''
            business_rules = utils.extract_exception_messages(func)
            post_condition = _('Action successfully performed')
            self.name = action_dict['verbose_name']
            self.description = description
            self.business_rules = business_rules
            self.post_condition = post_condition
            for can_execute in action_dict['can_execute']:
                self.actors.append(can_execute)
            if not self.actors:
                self.actors.append(_('Superuser'))
        elif hasattr(func, '_view_action'):
            button_label = func._view_action['verbose_name']
            interaction = _('The user clicks the button')
            self._interactions.append('{} "{}"'.format(interaction, button_label))
            self._test_function_code.append("        self.click_button('{}')".format(button_label))

    def _execute_view(self, action):
        interaction = _('The user clicks the button')
        self._interactions.append('{} "{}"'.format(interaction, action))
        self._test_function_code.append("        self.click_button('{}')".format(action))

    def get_interactions_as_string(self):
        l = []
        for i, interaction in enumerate(self._interactions):
            l.append('            {}. {}'.format(i+1, interaction))
        return '\n'.join(l)

    def get_test_function_signature(self):
        return self._func_signature

    def get_test_function_code(self):
        return '\n'.join(self._test_function_code)

    def as_dict(self):
        return dict(name=self.name, description=self.description, actors=self.actors,
                    business_rules=self.business_rules, pre_conditions=self.pre_conditions,
                    post_condition=self.post_condition, scenario=self.get_interactions_as_string())



# -*- coding: utf-8 -*-

ICONS = list()
ICONS.append('glass')
ICONS.append('music')
ICONS.append('search')
ICONS.append('envelope-o')
ICONS.append('heart')
ICONS.append('star')
ICONS.append('star-o')
ICONS.append('user')
ICONS.append('film')
ICONS.append('th-large')
ICONS.append('th')
ICONS.append('th-list')
ICONS.append('check')
ICONS.append('remove')
ICONS.append('close')
ICONS.append('times')
ICONS.append('search-plus')
ICONS.append('search-minus')
ICONS.append('power-off')
ICONS.append('signal')
ICONS.append('gear')
ICONS.append('cog')
ICONS.append('trash-o')
ICONS.append('home')
ICONS.append('file-o')
ICONS.append('clock-o')
ICONS.append('road')
ICONS.append('download')
ICONS.append('arrow-circle-o-down')
ICONS.append('arrow-circle-o-up')
ICONS.append('inbox')
ICONS.append('play-circle-o')
ICONS.append('rotate-right')
ICONS.append('repeat')
ICONS.append('refresh')
ICONS.append('list-alt')
ICONS.append('lock')
ICONS.append('flag')
ICONS.append('headphones')
ICONS.append('volume-off')
ICONS.append('volume-down')
ICONS.append('volume-up')
ICONS.append('qrcode')
ICONS.append('barcode')
ICONS.append('tag')
ICONS.append('tags')
ICONS.append('book')
ICONS.append('bookmark')
ICONS.append('print')
ICONS.append('camera')
ICONS.append('font')
ICONS.append('bold')
ICONS.append('italic')
ICONS.append('text-height')
ICONS.append('text-width')
ICONS.append('align-left')
ICONS.append('align-center')
ICONS.append('align-right')
ICONS.append('align-justify')
ICONS.append('list')
ICONS.append('dedent')
ICONS.append('outdent')
ICONS.append('indent')
ICONS.append('video-camera')
ICONS.append('photo')
ICONS.append('image')
ICONS.append('picture-o')
ICONS.append('pencil')
ICONS.append('map-marker')
ICONS.append('adjust')
ICONS.append('tint')
ICONS.append('edit')
ICONS.append('pencil-square-o')
ICONS.append('share-square-o')
ICONS.append('check-square-o')
ICONS.append('arrows')
ICONS.append('step-backward')
ICONS.append('fast-backward')
ICONS.append('backward')
ICONS.append('play')
ICONS.append('pause')
ICONS.append('stop')
ICONS.append('forward')
ICONS.append('fast-forward')
ICONS.append('step-forward')
ICONS.append('eject')
ICONS.append('chevron-left')
ICONS.append('chevron-right')
ICONS.append('plus-circle')
ICONS.append('minus-circle')
ICONS.append('times-circle')
ICONS.append('check-circle')
ICONS.append('question-circle')
ICONS.append('info-circle')
ICONS.append('crosshairs')
ICONS.append('times-circle-o')
ICONS.append('check-circle-o')
ICONS.append('ban')
ICONS.append('arrow-left')
ICONS.append('arrow-right')
ICONS.append('arrow-up')
ICONS.append('arrow-down')
ICONS.append('mail-forward')
ICONS.append('share')
ICONS.append('expand')
ICONS.append('compress')
ICONS.append('plus')
ICONS.append('minus')
ICONS.append('asterisk')
ICONS.append('exclamation-circle')
ICONS.append('gift')
ICONS.append('leaf')
ICONS.append('fire')
ICONS.append('eye')
ICONS.append('eye-slash')
ICONS.append('warning')
ICONS.append('exclamation-triangle')
ICONS.append('plane')
ICONS.append('calendar')
ICONS.append('random')
ICONS.append('comment')
ICONS.append('magnet')
ICONS.append('chevron-up')
ICONS.append('chevron-down')
ICONS.append('retweet')
ICONS.append('shopping-cart')
ICONS.append('folder')
ICONS.append('folder-open')
ICONS.append('arrows-v')
ICONS.append('arrows-h')
ICONS.append('bar-chart-o')
ICONS.append('bar-chart')
ICONS.append('twitter-square')
ICONS.append('facebook-square')
ICONS.append('camera-retro')
ICONS.append('key')
ICONS.append('gears')
ICONS.append('cogs')
ICONS.append('comments')
ICONS.append('thumbs-o-up')
ICONS.append('thumbs-o-down')
ICONS.append('star-half')
ICONS.append('heart-o')
ICONS.append('sign-out')
ICONS.append('linkedin-square')
ICONS.append('thumb-tack')
ICONS.append('external-link')
ICONS.append('sign-in')
ICONS.append('trophy')
ICONS.append('github-square')
ICONS.append('upload')
ICONS.append('lemon-o')
ICONS.append('phone')
ICONS.append('square-o')
ICONS.append('bookmark-o')
ICONS.append('phone-square')
ICONS.append('twitter')
ICONS.append('facebook-f')
ICONS.append('facebook')
ICONS.append('github')
ICONS.append('unlock')
ICONS.append('credit-card')
ICONS.append('feed')
ICONS.append('rss')
ICONS.append('hdd-o')
ICONS.append('bullhorn')
ICONS.append('bell')
ICONS.append('certificate')
ICONS.append('hand-o-right')
ICONS.append('hand-o-left')
ICONS.append('hand-o-up')
ICONS.append('hand-o-down')
ICONS.append('arrow-circle-left')
ICONS.append('arrow-circle-right')
ICONS.append('arrow-circle-up')
ICONS.append('arrow-circle-down')
ICONS.append('globe')
ICONS.append('wrench')
ICONS.append('tasks')
ICONS.append('filter')
ICONS.append('briefcase')
ICONS.append('arrows-alt')
ICONS.append('group')
ICONS.append('users')
ICONS.append('chain')
ICONS.append('link')
ICONS.append('cloud')
ICONS.append('flask')
ICONS.append('cut')
ICONS.append('scissors')
ICONS.append('copy')
ICONS.append('files-o')
ICONS.append('paperclip')
ICONS.append('save')
ICONS.append('floppy-o')
ICONS.append('square')
ICONS.append('navicon')
ICONS.append('reorder')
ICONS.append('bars')
ICONS.append('list-ul')
ICONS.append('list-ol')
ICONS.append('strikethrough')
ICONS.append('underline')
ICONS.append('table')
ICONS.append('magic')
ICONS.append('truck')
ICONS.append('pinterest')
ICONS.append('pinterest-square')
ICONS.append('google-plus-square')
ICONS.append('google-plus')
ICONS.append('money')
ICONS.append('caret-down')
ICONS.append('caret-up')
ICONS.append('caret-left')
ICONS.append('caret-right')
ICONS.append('columns')
ICONS.append('unsorted')
ICONS.append('sort')
ICONS.append('sort-down')
ICONS.append('sort-desc')
ICONS.append('sort-up')
ICONS.append('sort-asc')
ICONS.append('envelope')
ICONS.append('linkedin')
ICONS.append('rotate-left')
ICONS.append('undo')
ICONS.append('legal')
ICONS.append('gavel')
ICONS.append('dashboard')
ICONS.append('tachometer')
ICONS.append('comment-o')
ICONS.append('comments-o')
ICONS.append('flash')
ICONS.append('bolt')
ICONS.append('sitemap')
ICONS.append('umbrella')
ICONS.append('paste')
ICONS.append('clipboard')
ICONS.append('lightbulb-o')
ICONS.append('exchange')
ICONS.append('cloud-download')
ICONS.append('cloud-upload')
ICONS.append('user-md')
ICONS.append('stethoscope')
ICONS.append('suitcase')
ICONS.append('bell-o')
ICONS.append('coffee')
ICONS.append('cutlery')
ICONS.append('file-text-o')
ICONS.append('building-o')
ICONS.append('hospital-o')
ICONS.append('ambulance')
ICONS.append('medkit')
ICONS.append('fighter-jet')
ICONS.append('beer')
ICONS.append('h-square')
ICONS.append('plus-square')
ICONS.append('angle-double-left')
ICONS.append('angle-double-right')
ICONS.append('angle-double-up')
ICONS.append('angle-double-down')
ICONS.append('angle-left')
ICONS.append('angle-right')
ICONS.append('angle-up')
ICONS.append('angle-down')
ICONS.append('desktop')
ICONS.append('laptop')
ICONS.append('tablet')
ICONS.append('mobile-phone')
ICONS.append('mobile')
ICONS.append('circle-o')
ICONS.append('quote-left')
ICONS.append('quote-right')
ICONS.append('spinner')
ICONS.append('circle')
ICONS.append('mail-reply')
ICONS.append('reply')
ICONS.append('github-alt')
ICONS.append('folder-o')
ICONS.append('folder-open-o')
ICONS.append('smile-o')
ICONS.append('frown-o')
ICONS.append('meh-o')
ICONS.append('gamepad')
ICONS.append('keyboard-o')
ICONS.append('flag-o')
ICONS.append('flag-checkered')
ICONS.append('terminal')
ICONS.append('code')
ICONS.append('mail-reply-all')
ICONS.append('reply-all')
ICONS.append('star-half-empty')
ICONS.append('star-half-full')
ICONS.append('star-half-o')
ICONS.append('location-arrow')
ICONS.append('crop')
ICONS.append('code-fork')
ICONS.append('unlink')
ICONS.append('chain-broken')
ICONS.append('question')
ICONS.append('info')
ICONS.append('exclamation')
ICONS.append('superscript')
ICONS.append('subscript')
ICONS.append('eraser')
ICONS.append('puzzle-piece')
ICONS.append('microphone')
ICONS.append('microphone-slash')
ICONS.append('shield')
ICONS.append('calendar-o')
ICONS.append('fire-extinguisher')
ICONS.append('rocket')
ICONS.append('maxcdn')
ICONS.append('chevron-circle-left')
ICONS.append('chevron-circle-right')
ICONS.append('chevron-circle-up')
ICONS.append('chevron-circle-down')
ICONS.append('html5')
ICONS.append('css3')
ICONS.append('anchor')
ICONS.append('unlock-alt')
ICONS.append('bullseye')
ICONS.append('ellipsis-h')
ICONS.append('ellipsis-v')
ICONS.append('rss-square')
ICONS.append('play-circle')
ICONS.append('ticket')
ICONS.append('minus-square')
ICONS.append('minus-square-o')
ICONS.append('level-up')
ICONS.append('level-down')
ICONS.append('check-square')
ICONS.append('pencil-square')
ICONS.append('external-link-square')
ICONS.append('share-square')
ICONS.append('compass')
ICONS.append('toggle-down')
ICONS.append('caret-square-o-down')
ICONS.append('toggle-up')
ICONS.append('caret-square-o-up')
ICONS.append('toggle-right')
ICONS.append('caret-square-o-right')
ICONS.append('euro')
ICONS.append('eur')
ICONS.append('gbp')
ICONS.append('dollar')
ICONS.append('usd')
ICONS.append('rupee')
ICONS.append('inr')
ICONS.append('cny')
ICONS.append('rmb')
ICONS.append('yen')
ICONS.append('jpy')
ICONS.append('ruble')
ICONS.append('rouble')
ICONS.append('rub')
ICONS.append('won')
ICONS.append('krw')
ICONS.append('bitcoin')
ICONS.append('btc')
ICONS.append('file')
ICONS.append('file-text')
ICONS.append('sort-alpha-asc')
ICONS.append('sort-alpha-desc')
ICONS.append('sort-amount-asc')
ICONS.append('sort-amount-desc')
ICONS.append('sort-numeric-asc')
ICONS.append('sort-numeric-desc')
ICONS.append('thumbs-up')
ICONS.append('thumbs-down')
ICONS.append('youtube-square')
ICONS.append('youtube')
ICONS.append('xing')
ICONS.append('xing-square')
ICONS.append('youtube-play')
ICONS.append('dropbox')
ICONS.append('stack-overflow')
ICONS.append('instagram')
ICONS.append('flickr')
ICONS.append('adn')
ICONS.append('bitbucket')
ICONS.append('bitbucket-square')
ICONS.append('tumblr')
ICONS.append('tumblr-square')
ICONS.append('long-arrow-down')
ICONS.append('long-arrow-up')
ICONS.append('long-arrow-left')
ICONS.append('long-arrow-right')
ICONS.append('apple')
ICONS.append('windows')
ICONS.append('android')
ICONS.append('linux')
ICONS.append('dribbble')
ICONS.append('skype')
ICONS.append('foursquare')
ICONS.append('trello')
ICONS.append('female')
ICONS.append('male')
ICONS.append('gittip')
ICONS.append('gratipay')
ICONS.append('sun-o')
ICONS.append('moon-o')
ICONS.append('archive')
ICONS.append('bug')
ICONS.append('vk')
ICONS.append('weibo')
ICONS.append('renren')
ICONS.append('pagelines')
ICONS.append('stack-exchange')
ICONS.append('arrow-circle-o-right')
ICONS.append('arrow-circle-o-left')
ICONS.append('toggle-left')
ICONS.append('caret-square-o-left')
ICONS.append('dot-circle-o')
ICONS.append('wheelchair')
ICONS.append('vimeo-square')
ICONS.append('turkish-lira')
ICONS.append('try')
ICONS.append('plus-square-o')
ICONS.append('space-shuttle')
ICONS.append('slack')
ICONS.append('envelope-square')
ICONS.append('wordpress')
ICONS.append('openid')
ICONS.append('institution')
ICONS.append('bank')
ICONS.append('university')
ICONS.append('mortar-board')
ICONS.append('graduation-cap')
ICONS.append('yahoo')
ICONS.append('google')
ICONS.append('reddit')
ICONS.append('reddit-square')
ICONS.append('stumbleupon-circle')
ICONS.append('stumbleupon')
ICONS.append('delicious')
ICONS.append('digg')
ICONS.append('pied-piper-pp')
ICONS.append('pied-piper-alt')
ICONS.append('drupal')
ICONS.append('joomla')
ICONS.append('language')
ICONS.append('fax')
ICONS.append('building')
ICONS.append('child')
ICONS.append('paw')
ICONS.append('spoon')
ICONS.append('cube')
ICONS.append('cubes')
ICONS.append('behance')
ICONS.append('behance-square')
ICONS.append('steam')
ICONS.append('steam-square')
ICONS.append('recycle')
ICONS.append('automobile')
ICONS.append('car')
ICONS.append('cab')
ICONS.append('taxi')
ICONS.append('tree')
ICONS.append('spotify')
ICONS.append('deviantart')
ICONS.append('soundcloud')
ICONS.append('database')
ICONS.append('file-pdf-o')
ICONS.append('file-word-o')
ICONS.append('file-excel-o')
ICONS.append('file-powerpoint-o')
ICONS.append('file-photo-o')
ICONS.append('file-picture-o')
ICONS.append('file-image-o')
ICONS.append('file-zip-o')
ICONS.append('file-archive-o')
ICONS.append('file-sound-o')
ICONS.append('file-audio-o')
ICONS.append('file-movie-o')
ICONS.append('file-video-o')
ICONS.append('file-code-o')
ICONS.append('vine')
ICONS.append('codepen')
ICONS.append('jsfiddle')
ICONS.append('life-bouy')
ICONS.append('life-buoy')
ICONS.append('life-saver')
ICONS.append('support')
ICONS.append('life-ring')
ICONS.append('circle-o-notch')
ICONS.append('ra')
ICONS.append('resistance')
ICONS.append('rebel')
ICONS.append('ge')
ICONS.append('empire')
ICONS.append('git-square')
ICONS.append('git')
ICONS.append('y-combinator-square')
ICONS.append('yc-square')
ICONS.append('hacker-news')
ICONS.append('tencent-weibo')
ICONS.append('qq')
ICONS.append('wechat')
ICONS.append('weixin')
ICONS.append('send')
ICONS.append('paper-plane')
ICONS.append('send-o')
ICONS.append('paper-plane-o')
ICONS.append('history')
ICONS.append('circle-thin')
ICONS.append('header')
ICONS.append('paragraph')
ICONS.append('sliders')
ICONS.append('share-alt')
ICONS.append('share-alt-square')
ICONS.append('bomb')
ICONS.append('soccer-ball-o')
ICONS.append('futbol-o')
ICONS.append('tty')
ICONS.append('binoculars')
ICONS.append('plug')
ICONS.append('slideshare')
ICONS.append('twitch')
ICONS.append('yelp')
ICONS.append('newspaper-o')
ICONS.append('wifi')
ICONS.append('calculator')
ICONS.append('paypal')
ICONS.append('google-wallet')
ICONS.append('cc-visa')
ICONS.append('cc-mastercard')
ICONS.append('cc-discover')
ICONS.append('cc-amex')
ICONS.append('cc-paypal')
ICONS.append('cc-stripe')
ICONS.append('bell-slash')
ICONS.append('bell-slash-o')
ICONS.append('trash')
ICONS.append('copyright')
ICONS.append('at')
ICONS.append('eyedropper')
ICONS.append('paint-brush')
ICONS.append('birthday-cake')
ICONS.append('area-chart')
ICONS.append('pie-chart')
ICONS.append('line-chart')
ICONS.append('lastfm')
ICONS.append('lastfm-square')
ICONS.append('toggle-off')
ICONS.append('toggle-on')
ICONS.append('bicycle')
ICONS.append('bus')
ICONS.append('ioxhost')
ICONS.append('angellist')
ICONS.append('cc')
ICONS.append('shekel')
ICONS.append('sheqel')
ICONS.append('ils')
ICONS.append('meanpath')
ICONS.append('buysellads')
ICONS.append('connectdevelop')
ICONS.append('dashcube')
ICONS.append('forumbee')
ICONS.append('leanpub')
ICONS.append('sellsy')
ICONS.append('shirtsinbulk')
ICONS.append('simplybuilt')
ICONS.append('skyatlas')
ICONS.append('cart-plus')
ICONS.append('cart-arrow-down')
ICONS.append('diamond')
ICONS.append('ship')
ICONS.append('user-secret')
ICONS.append('motorcycle')
ICONS.append('street-view')
ICONS.append('heartbeat')
ICONS.append('venus')
ICONS.append('mars')
ICONS.append('mercury')
ICONS.append('intersex')
ICONS.append('transgender')
ICONS.append('transgender-alt')
ICONS.append('venus-double')
ICONS.append('mars-double')
ICONS.append('venus-mars')
ICONS.append('mars-stroke')
ICONS.append('mars-stroke-v')
ICONS.append('mars-stroke-h')
ICONS.append('neuter')
ICONS.append('genderless')
ICONS.append('facebook-official')
ICONS.append('pinterest-p')
ICONS.append('whatsapp')
ICONS.append('server')
ICONS.append('user-plus')
ICONS.append('user-times')
ICONS.append('hotel')
ICONS.append('bed')
ICONS.append('viacoin')
ICONS.append('train')
ICONS.append('subway')
ICONS.append('medium')
ICONS.append('yc')
ICONS.append('y-combinator')
ICONS.append('optin-monster')
ICONS.append('opencart')
ICONS.append('expeditedssl')
ICONS.append('battery-4')
ICONS.append('battery')
ICONS.append('battery-full')
ICONS.append('battery-3')
ICONS.append('battery-three-quarters')
ICONS.append('battery-2')
ICONS.append('battery-half')
ICONS.append('battery-1')
ICONS.append('battery-quarter')
ICONS.append('battery-0')
ICONS.append('battery-empty')
ICONS.append('mouse-pointer')
ICONS.append('i-cursor')
ICONS.append('object-group')
ICONS.append('object-ungroup')
ICONS.append('sticky-note')
ICONS.append('sticky-note-o')
ICONS.append('cc-jcb')
ICONS.append('cc-diners-club')
ICONS.append('clone')
ICONS.append('balance-scale')
ICONS.append('hourglass-o')
ICONS.append('hourglass-1')
ICONS.append('hourglass-start')
ICONS.append('hourglass-2')
ICONS.append('hourglass-half')
ICONS.append('hourglass-3')
ICONS.append('hourglass-end')
ICONS.append('hourglass')
ICONS.append('hand-grab-o')
ICONS.append('hand-rock-o')
ICONS.append('hand-stop-o')
ICONS.append('hand-paper-o')
ICONS.append('hand-scissors-o')
ICONS.append('hand-lizard-o')
ICONS.append('hand-spock-o')
ICONS.append('hand-pointer-o')
ICONS.append('hand-peace-o')
ICONS.append('trademark')
ICONS.append('registered')
ICONS.append('creative-commons')
ICONS.append('gg')
ICONS.append('gg-circle')
ICONS.append('tripadvisor')
ICONS.append('odnoklassniki')
ICONS.append('odnoklassniki-square')
ICONS.append('get-pocket')
ICONS.append('wikipedia-w')
ICONS.append('safari')
ICONS.append('chrome')
ICONS.append('firefox')
ICONS.append('opera')
ICONS.append('internet-explorer')
ICONS.append('tv')
ICONS.append('television')
ICONS.append('contao')
ICONS.append('500px')
ICONS.append('amazon')
ICONS.append('calendar-plus-o')
ICONS.append('calendar-minus-o')
ICONS.append('calendar-times-o')
ICONS.append('calendar-check-o')
ICONS.append('industry')
ICONS.append('map-pin')
ICONS.append('map-signs')
ICONS.append('map-o')
ICONS.append('map')
ICONS.append('commenting')
ICONS.append('commenting-o')
ICONS.append('houzz')
ICONS.append('vimeo')
ICONS.append('black-tie')
ICONS.append('fonticons')
ICONS.append('reddit-alien')
ICONS.append('edge')
ICONS.append('credit-card-alt')
ICONS.append('codiepie')
ICONS.append('modx')
ICONS.append('fort-awesome')
ICONS.append('usb')
ICONS.append('product-hunt')
ICONS.append('mixcloud')
ICONS.append('scribd')
ICONS.append('pause-circle')
ICONS.append('pause-circle-o')
ICONS.append('stop-circle')
ICONS.append('stop-circle-o')
ICONS.append('shopping-bag')
ICONS.append('shopping-basket')
ICONS.append('hashtag')
ICONS.append('bluetooth')
ICONS.append('bluetooth-b')
ICONS.append('percent')
ICONS.append('gitlab')
ICONS.append('wpbeginner')
ICONS.append('wpforms')
ICONS.append('envira')
ICONS.append('universal-access')
ICONS.append('wheelchair-alt')
ICONS.append('question-circle-o')
ICONS.append('blind')
ICONS.append('audio-description')
ICONS.append('volume-control-phone')
ICONS.append('braille')
ICONS.append('assistive-listening-systems')
ICONS.append('asl-interpreting')
ICONS.append('american-sign-language-interpreting')
ICONS.append('deafness')
ICONS.append('hard-of-hearing')
ICONS.append('deaf')
ICONS.append('glide')
ICONS.append('glide-g')
ICONS.append('signing')
ICONS.append('sign-language')
ICONS.append('low-vision')
ICONS.append('viadeo')
ICONS.append('viadeo-square')
ICONS.append('snapchat')
ICONS.append('snapchat-ghost')
ICONS.append('snapchat-square')
ICONS.append('pied-piper')
ICONS.append('first-order')
ICONS.append('yoast')
ICONS.append('themeisle')
ICONS.append('google-plus-circle')
ICONS.append('google-plus-official')
ICONS.append('fa')
ICONS.append('font-awesome')
ICONS.append('handshake-o')
ICONS.append('envelope-open')
ICONS.append('envelope-open-o')
ICONS.append('linode')
ICONS.append('address-book')
ICONS.append('address-book-o')
ICONS.append('vcard')
ICONS.append('address-card')
ICONS.append('vcard-o')
ICONS.append('address-card-o')
ICONS.append('user-circle')
ICONS.append('user-circle-o')
ICONS.append('user-o')
ICONS.append('id-badge')
ICONS.append('drivers-license')
ICONS.append('id-card')
ICONS.append('drivers-license-o')
ICONS.append('id-card-o')
ICONS.append('quora')
ICONS.append('free-code-camp')
ICONS.append('telegram')
ICONS.append('thermometer-4')
ICONS.append('thermometer')
ICONS.append('thermometer-full')
ICONS.append('thermometer-3')
ICONS.append('thermometer-three-quarters')
ICONS.append('thermometer-2')
ICONS.append('thermometer-half')
ICONS.append('thermometer-1')
ICONS.append('thermometer-quarter')
ICONS.append('thermometer-0')
ICONS.append('thermometer-empty')
ICONS.append('shower')
ICONS.append('bathtub')
ICONS.append('s15')
ICONS.append('bath')
ICONS.append('podcast')
ICONS.append('window-maximize')
ICONS.append('window-minimize')
ICONS.append('window-restore')
ICONS.append('times-rectangle')
ICONS.append('window-close')
ICONS.append('times-rectangle-o')
ICONS.append('window-close-o')
ICONS.append('bandcamp')
ICONS.append('grav')
ICONS.append('etsy')
ICONS.append('imdb')
ICONS.append('ravelry')
ICONS.append('eercast')
ICONS.append('microchip')
ICONS.append('snowflake-o')
ICONS.append('superpowers')
ICONS.append('wpexplorer')
ICONS.append('meetup')

ICON_CHOICES = [['<i class="fa fa-{} fa-fw"></i>'.format(icon_name), icon_name] for icon_name in ICONS]



# -*- coding: utf-8 -*-

from djangoplus.mail.utils import dump_emails
from django.core.mail.backends.base import BaseEmailBackend


class EmailDebugBackend(BaseEmailBackend):

    def send_messages(self, email_messages):
        return dump_emails(email_messages)



# -*- coding: utf-8 -*-

import copy
from datetime import datetime
from djangoplus.utils import http
from collections import OrderedDict
from djangoplus.ui.components import Component
from django.utils.translation import ugettext as _
from djangoplus.utils.metadata import check_condition
from djangoplus.utils import permissions, should_add_action
from djangoplus.utils.metadata import get_metadata, get_can_execute, count_parameters_names


class GroupDropDown(Component):
    def __init__(self, request, style=None):
        super(GroupDropDown, self).__init__('groupdropdown', request)
        self.actions = OrderedDict()
        self.mobile = http.mobile(self.request)
        self.actions[_('Actions')] = []
        self.style = style

    def add_action(self, label, url, css='popup', icon=None, category=None):
        if category is None:
            category = label
        if self.mobile:
            category = _('Actions')
        if category not in self.actions:
            self.actions[category] = []
        item = dict(label=label, url=url, css=css, icon=icon)
        self.actions[category].append(item)

    def has_items(self):
        for category in self.actions:
            if self.actions[category]:
                return True
        return False


class ModelDropDown(GroupDropDown):
    def __init__(self, request, model, action_names=()):

        self.model = model
        self.action_names = action_names
        self.actions_cache = None
        self.hash = abs(datetime.now().__hash__())
        self.obj = None
        self.has_inline_action = False

        super(ModelDropDown, self).__init__(request)

        self.load_actions()

    def load_actions(self):
        from djangoplus.cache import CACHE
        if self.action_names and len(self.action_names) > 0:
            self.has_inline_action = True

        for category in CACHE['INSTANCE_ACTIONS'][self.model]:
            if category not in self.actions:
                self.actions[category] = []
            for view_name in CACHE['INSTANCE_ACTIONS'][self.model][category]:
                form_action = CACHE['INSTANCE_ACTIONS'][self.model][category][view_name]
                if not self.has_inline_action:
                    self.has_inline_action = form_action.get('inline') or form_action.get('subsets')

    def add_action(self, label, url, css='popup', icon=None, category=None):
        if category is None:
            category = _('Actions')
        if category not in self.actions:
            self.actions[category] = []
        if self.obj:
            if '{}' in url:
                url = url.format(self.obj.pk)
            else:
                url = '{}{}/'.format(url, self.obj.pk)
        super(ModelDropDown, self).add_action(label, url, css, icon, category)

    def add_actions(self, obj, fieldset=None, subset_name=None, category=None, action_names=None):
        from djangoplus.cache import CACHE
        obj.request = self.request
        if not self.actions_cache:
            self.actions_cache = self.actions
        if not action_names:
            action_names = []
        self.hash = abs(datetime.now().__hash__())
        self.actions = copy.deepcopy(self.actions_cache)
        self.obj = obj

        for action_category in CACHE['INSTANCE_ACTIONS'][self.model]:

            for view_name in CACHE['INSTANCE_ACTIONS'][self.model][action_category]:
                action = CACHE['INSTANCE_ACTIONS'][self.model][action_category][view_name]
                action_function = action.get('function')
                action_verbose_name = action['verbose_name']
                action_can_execute = get_can_execute(action)
                action_condition = action['condition']
                action_style = action['style']
                action_input = action['input']
                action_inline = action['inline']
                action_subsets = action['subsets']
                action_icon = action['icon']
                action_display = action['display']
                action_expose = action['expose']

                action_name = action_function.__name__
                is_action_view = not hasattr(self.model, action_name)

                if action_name and action_style == 'popup' and not is_action_view:
                    func = getattr(self.model, action_name)
                    if not (count_parameters_names(func) > 1 or action_input or action_display):
                        action_style = 'ajax'

                # it is a dropdown in a model panel
                if fieldset is not None:
                    if fieldset:
                        if action_name not in CACHE['FIELDSET_ACTIONS'][self.model][fieldset]:
                            continue
                    else:
                        # if the action was included in any fieldset,
                        # it can not be displayed in page's action panel
                        add_action = True
                        for title, action_names in list(CACHE['FIELDSET_ACTIONS'][self.model].items()):
                            if action_name in action_names:
                                add_action = False
                                break
                        if not add_action:
                            continue
                else:
                    # it is a dropdown in a paginator
                    if action_names:
                        # it is a relation paginator
                        if view_name not in action_names:
                            continue
                    else:
                        # it is a list view paginator
                        should_add_this_action = should_add_action(action_inline, action_subsets, subset_name)
                        add_action = view_name in action_names or should_add_this_action
                        if not add_action:
                            continue

                lookups = self.request.user.get_permission_mapping(self.model, obj).get(action_name)
                if lookups:
                    forbidden = True
                    for key, value in lookups:
                        if self.model.objects.filter(pk=obj.pk, **{'{}__in'.format(key): value}).exists():
                            forbidden = False
                            break
                    if forbidden:
                        continue

                if True not in action_expose and 'web' not in action_expose:
                    continue

                if not permissions.check_group_or_permission(self.request, action_can_execute):
                    continue

                if not check_condition(self.request.user, action_condition, obj):
                    continue

                if is_action_view:
                    action_url = '/{}/{}/'.format(get_metadata(self.model, 'app_label'), action_name)
                else:
                    action_url = '/action/{}/{}/{}/'.format(
                        get_metadata(self.model, 'app_label'), self.model.__name__.lower(), view_name
                    )

                action_category = category or action_category
                self.add_action(action_verbose_name, action_url, action_style, action_icon, action_category)



# -*- coding: utf-8 -*-

from django.core.management.commands import migrate
from djangoplus.admin.management import sync_permissions


class Command(migrate.Command):

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--ignore_sync_permissions', action='store_true', dest='ignore_sync_permissions')

    def handle(self, *args, **options):
        super(Command, self).handle(*args, **options)
        if not options.get('ignore_sync_permissions'):
            sync_permissions()



# -*- coding: utf-8 -*-
from django.core.management.base import BaseCommand
from djangoplus.tools.video import VideoRecorder
from djangoplus.tools.subtitle import Subtitle
from djangoplus.tools.browser import Browser


class Command(BaseCommand):

    def handle(self, *args, **options):
        recorder = VideoRecorder()
        browser = Browser('http://google.com.br')
        browser.open('/')
        recorder.start()
        Subtitle.display('This is only a test', 3)
        recorder.stop('Test')
        browser.close()
        browser.service.stop()



# -*- coding: utf-8 -*-

from django.conf import settings
from django.utils import translation
from djangoplus.tools import terminal
from djangoplus.docs import Documentation
from django.utils.translation import ugettext as _
from django.core.management.base import BaseCommand


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--usecase', action='store', nargs=1, dest='usecase', default=False,
                            help='Prints the documetation of a specific usecase')
        parser.add_argument('--json', action='store_true', dest='json', default=False, help='')

    def handle(self, *args, **options):

        doc = Documentation()
        translation.activate(settings.LANGUAGE_CODE)

        usecase_name = options.get('usecase')

        if options.pop('json', False):
            print(str(doc.as_json()))
        else:
            output = list()
            if not usecase_name:
                output.append(terminal.bold(_('Description:').upper()))
                output.append(terminal.info(doc.description))
                output.append('')
                output.append(terminal.bold(_('Actors:').upper()))
                for i, actor in enumerate(doc.actors):
                    output.append('{}. {}'.format(i + 1, actor.name))
                output.append('')
                output.append(terminal.bold(_('Workflow:').upper()))
                for i, task in enumerate(doc.workflow.tasks):
                    output.append('{} {}'.format(' ' * i, task))
                output.append('')
            output.append(terminal.bold(_('Usecases:').upper()))
            for i, usecase in enumerate(doc.usecases):
                if not usecase_name or usecase.name == usecase_name[0]:
                    output.append(terminal.bold('* Usecase #{}'.format((i + 1))))
                    output.append('')
                    output.append('{}:\t\t\t{}'.format(terminal.info(_('Name')), usecase.name))
                    output.append('{}:\t\t{}'.format(terminal.info(_('Description')), usecase.description or ''))
                    output.append('{}:\t\t\t{}'.format(terminal.info(_('Actors')), ', '.join(usecase.actors)))
                    output.append('{}:\t\t{}'.format(terminal.info(_('Buniness Rules')), ', '.join(usecase.business_rules)))
                    output.append('{}:\t\t{}'.format(terminal.info(_('Pre-conditions')), ', '.join(usecase.pre_conditions)))
                    output.append('{}:\t\t{}'.format(terminal.info(_('Post-condition')), (usecase.post_condition or '')))
                    output.append('{}:'.format(terminal.info(_('Main-scenario'))))
                    output.append(usecase.get_interactions_as_string())
                    output.append('')
            output.append('')
            if not usecase_name:
                output.append(terminal.bold(_('Class Diagrams:').upper()))
                for class_diagram in doc.class_diagrams:
                    output.append(terminal.bold('\t{} {}'.format(class_diagram.name, _('Diagram'))))
                    for i, cls in enumerate(class_diagram.classes):
                        output.append('\t\t\t{}. {}'.format(i + 1, cls['name']))
            return '\n'.join(output)



# -*- coding: utf-8 -*-

# Generated by Django 1.10 on 2016-11-24 18:25


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0007_auto_20161124_1811'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='is_superuser',
            field=models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status'),
        ),
        migrations.AlterField(
            model_name='user',
            name='password',
            field=models.CharField(max_length=128, verbose_name='password'),
        ),
    ]



# -*- coding: utf-8 -*-

from djangoplus.ui.components.navigation.breadcrumbs import Breadcrumbs
from django_jinja import library as register


@register.global_function
def breadcrumbs(request, view_title):
    return str(Breadcrumbs(request, view_title))



# -*- coding: utf-8 -*-
import json
from os import path, listdir
from django.conf import settings
from djangoplus.cache import CACHE
from djangoplus.admin.models import Group
from djangoplus.docs import Documentation, ApiDocumentation
from djangoplus.decorators.views import view, action
from djangoplus.admin.models import User, Organization
from djangoplus.tools.video import VideoUploader


@view('Source', login_required=False)
def source(request, module=None):
    src = []
    if module == 'model':
        content = open(path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'models.py')).read()
        src.append(dict(file='models.py', language='python', content=content))
        content = open(path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'formatters.py')).read()
        src.append(dict(file='formatters.py', language='python', content=content))
        for template_file_name in listdir(path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'templates')):
            if template_file_name != 'public.html':
                content = open(path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'templates', template_file_name)).read()
                src.append(dict(file=template_file_name, language='html', content=content))
    elif module == 'tests':
        content = open(path.join(settings.BASE_DIR, settings.PROJECT_NAME, 'tests.py')).read()
        src.append(dict(file='tests.py', language='python', content=content))
    return locals()


@view('Homologação', login_required=False)
def homologate(request):
    title = 'Homologação'
    organization_model_name = None
    unit_model_name = None
    groups = Group.objects.exclude(name__in=('Usuário', 'User')).filter(role__scope__isnull=True).order_by('name').distinct()

    http_host = request.META['HTTP_HOST']
    if CACHE['ORGANIZATION_MODEL']:
        organization_model_name = CACHE['ORGANIZATION_MODEL']._meta.verbose_name
    if CACHE['UNIT_MODEL']:
        unit_model_name = CACHE['UNIT_MODEL']._meta.verbose_name

    organization_group_names = []
    unit_group_names = []
    for model in CACHE['ROLE_MODELS']:
        name = CACHE['ROLE_MODELS'][model]['name']
        scope = CACHE['ROLE_MODELS'][model]['scope']
        if scope == organization_model_name:
            organization_group_names.append(name)
        if scope == unit_model_name:
            unit_group_names.append(name)

    organization_groups = Group.objects.filter(name__in=organization_group_names).order_by('name').distinct()
    unit_groups = Group.objects.filter(name__in=unit_group_names).order_by('name').distinct()

    organizations = []
    for organization in Organization.objects.all():

        organization.users = []
        for group in organization_groups:
            organization.users.append(User.objects.filter(role__group=group, role__scope=organization))

        organization.units = []
        for unit in organization.get_units():
            organization.units.append(unit)
            unit.users = []
            for group in unit_groups:
                unit.users.append(User.objects.filter(role__group=group, role__scope=unit))

        organizations.append(organization)
    return locals()


@view('Documentation', login_required=False)
def doc(request):
    documentation = Documentation()
    workflow_data = json.dumps(CACHE['WORKFLOWS'])
    return locals()


@view('Videos', login_required=False)
def tutorial(request):
    youtube = VideoUploader()
    videos = youtube.list_videos()
    return locals()


@view('API', login_required=False)
def api(request, app_label=None, model_name=None, endpoint_name=None):
    documentation = ApiDocumentation()
    endpoint = None
    token = None
    if app_label and model_name:
        endpoint = documentation.load_model(app_label, model_name, endpoint_name)
    else:
        documentation.load_models()
    if request.user.is_authenticated:
        token = request.user.token

    if endpoint:
        form_cls = documentation.form_cls(endpoint, token, request.META['HTTP_HOST'])
        form = form_cls(request)
        if form.is_valid():
            cmd, result = form.process()
            result = result.replace('\n', '<br>').replace(' ', '&nbsp')
    return locals()


@action(User, 'Login as User', can_execute='Superuser')
def login_as(request, pk):
    pass



# -*- coding: utf-8 -*-

import json
import datetime
from django.http.response import HttpResponse
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_exempt
from djangoplus.utils.serialization import loads_qs_query
from djangoplus.ui.components.paginator.templatetags import obj_icons
from djangoplus.ui.components.navigation.dropdown import ModelDropDown


@csrf_exempt
def populate(request):
    items = []
    queryset = loads_qs_query(request.POST['queryset'])
    start_field = request.POST['start_field']
    end_field = request.POST['end_field']
    start = datetime.datetime.strptime(request.POST['start'], '%Y-%m-%d')
    end = datetime.datetime.strptime(request.POST['end'], '%Y-%m-%d')
    fstart = start.strftime('%d/%m/%Y')
    fend = start.strftime('%d/%m/%Y')
    action_names = request.POST['action_names'].split(',')

    request.session['calendar_initial_date'] = (start + datetime.timedelta(days=10)).strftime('%Y-%m-%d')
    request.session.save()

    filters = dict()
    filters['{}__gte'.format(start_field)] = start
    filters['{}__lte'.format(start_field)] = end
    queryset = queryset.filter(**filters)

    for obj in queryset.all():
        title = str(obj)
        start = getattr(obj, start_field)
        end = end_field and end_field != 'None' and getattr(obj, end_field) or None
        if end:
            end = end + datetime.timedelta(days=1)
        icons = obj_icons(request, obj, css='popup')
        drop_down = ModelDropDown(
            request, obj.__class__, action_names=action_names
        )
        drop_down.add_actions(obj)
        html_id = hash(obj)
        html = list()
        html.append('<div id="{}">'.format(html_id))
        html.append('<div class="pull-right">{}</div><br/><br/>'.format(icons))

        if end:
            html.append('<dl>')
            html.append('<dt>{}</dt><dd>{}</dd>'.format(_('Start'), fstart))
            html.append('<dt>{}</dt><dd>{}</dd>'.format(_('End'), fend))
            html.append('</dl>')
        else:
            html.append('{}'.format(fstart))

        html.append('<hr/>')
        html.append(str(drop_down))
        html.append('</div>')
        html.append('<script>initialize("{}");</script>'.format(html_id))
        item = dict(
            id=html_id, title=title, start=str(start),
            end=end and str(end) or None, allDay=False,
            url='javascript:', html=''.join(html)
        )
        items.append(item)

    return HttpResponse(json.dumps(items))



# Generated by Django 2.1.7 on 2019-03-26 22:56

from django.db import migrations
import djangoplus.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0012_user_token'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='name',
            field=djangoplus.db.models.fields.CharField(blank=True, max_length=100, verbose_name='Nome'),
        ),
    ]



# -*- coding: utf-8 -*-

import tempfile
import qrcode
import base64
from decimal import Decimal
from django.conf import settings
from django.http import HttpResponse
from djangoplus.ui.components import forms
from djangoplus.utils.dateutils import DAY_NAMES
from djangoplus.utils.http import ComponentResponse
from djangoplus.ui.components import Component, ComponentHasResponseException
from djangoplus.utils.metadata import get_fiendly_name, get_field, get_metadata, getattr2


PERCENT_SYMBOL = '%'
DOLLAR_SYMBOL = '$'
REAIS_SYMBOL = 'R$'


class Chart(Component):

    def __init__(self, request, labels, series, groups=None, symbol=None, title=None, type=None):

        super(Chart, self).__init__(None, request)

        self.type = type or 'bar'
        self.labels = labels
        self.series = series
        self.groups = groups or []
        self.symbol = symbol or ''
        self.title = title
        self.color_index = 0
        self.display = False
        self.clickable = False

    def __str__(self):
        for i, serie in enumerate(self.series):
            for j, valor in enumerate(serie):
                if type(valor) == Decimal:
                    self.series[i][j] = float(valor)
                if valor:
                    self.display = True

        self.colors = []
        for color in settings.GRADIENT:
            self.colors.append(color)
        for i in range(len(self.colors), 16):
            self.colors.append('#FFFFFF')

        return super(Chart, self).__str__()

    def get_box_series(self):
        l = []
        serie = self.series[0]
        total = sum(serie)
        for i, label in enumerate(self.labels):
            percentage = int(total and serie[i]*100/total or 0)
            l.append((label, serie[i], percentage))
        return l

    def next_color(self):
        color = self.colors[self.color_index]
        self.color_index += 1
        return color

    def change_type(self, chart_type):
        self.type = chart_type
        return self

    def pie(self):
        return self.change_type('pie')

    def donut(self):
        return self.change_type('donut')

    def bar(self):
        return self.change_type('bar')

    def horizontal_bar(self):
        return self.change_type('horizontalbar')

    def stack(self):
        return self.change_type('stack')

    def horizontal_stack(self):
        return self.change_type('horizontalstack')

    def line(self):
        return self.change_type('line')

    def area(self):
        return self.change_type('area')

    def box(self):
        return self.change_type('box')


class QueryStatisticsChart(Chart):

    formatter_name = 'chart'
    template_name = 'chart.html'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None, type=None):

        groups = []
        labels = []
        series = []

        if queryset_statistics.groups:
            for label in queryset_statistics.labels:
                labels.append(label)
            for group in queryset_statistics.groups:
                groups.append(group)
        else:
            for label in queryset_statistics.labels:
                labels.append(label)

        for serie in queryset_statistics.series:
            series.append(serie)
    
        super(QueryStatisticsChart, self).__init__(request, labels, series, groups, symbol, title, type)

        self.clickable = self.request is not None
        self.queryset_statistics = queryset_statistics
        self.process_request()

    def process_request(self):
        super(Chart, self).process_request()
        if self.request and 'uuid' in self.request.GET and self.request.GET['uuid'] == self.title:
            note = None
            i, j = self.request.GET['position'].split('x')
            qs = self.queryset_statistics.querysets[int(i)][int(j)]
            title = get_metadata(qs.model, 'verbose_name_plural')
            if qs.count() > 25:
                qs = qs[0:25]
                note = u'Apenas uma amostra com 25 registros está sendo exibida.'
            component = ModelTable(self.request, title, qs, note=note)
            raise ComponentHasResponseException(ComponentResponse(component))


class QueryStatisticsLineChart(QueryStatisticsChart):
    formatter_name = 'line_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'line')


class QueryStatisticsAreaChart(QueryStatisticsChart):
    formatter_name = 'area_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'area')


class QueryStatisticsStackChart(QueryStatisticsChart):
    formatter_name = 'stack_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'stack')


class QueryStatisticsHorizontalStackChart(QueryStatisticsChart):
    formatter_name = 'horizontal_stack_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'horizontalstack')


class QueryStatisticsHorizontalBarChart(QueryStatisticsChart):
    formatter_name = 'horizontal_bar_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'horizontalbar')


class QueryStatisticsBarChart(QueryStatisticsChart):
    formatter_name = 'bar_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'bar')


class QueryStatisticsBoxChart(QueryStatisticsChart):
    formatter_name = 'box_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'box')


class QueryStatisticsDonutChart(QueryStatisticsChart):
    formatter_name = 'donut_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None):
        super().__init__(queryset_statistics, request, title, icon, symbol, 'donut')


class QueryStatisticsPieChart(QueryStatisticsChart):
    formatter_name = 'pie_chart'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None, symbol=None, type='pie'):
        super().__init__(queryset_statistics, request, title, icon, symbol)


class Timeline(Component):

    class Media:
        css = {'all': ('/static/css/timeline.css',)}

    def __init__(self, items, request=None, title=None):
        super(Timeline, self).__init__(title, request)
        self.items = items or []
        self.title = title
        self.width = 0

    def add(self, status, date):
        self.items.append((status, date))

    def __str__(self):
        self.width = str(100.0/len(self.items))
        return self.render('timeline.html')

class VerticalTimeline(Component):
    formatter_name = 'vertical_timeline'

    def __init__(self, items, request=None, title=None):
        super().__init__(title, request)
        self.items = items or []
        self.title = title

    def add(self, title, date, detail, url):
        self.items.append((title, date, detail, url))

class RoleSelector(Component):

    def __init__(self, request):
        super(RoleSelector, self).__init__('roleselector', request)
        self.groups = []
        self.process_request()
        self.display_button = False
        self.scopes = request.user.is_authenticated and request.user.get_appliable_scopes() or []
        self._load()

    def _load(self):
        if self.request.user.is_authenticated:
            from djangoplus.admin.models import Group, Role
            pks = self.request.user.role_set.values_list('group', flat=True).distinct()
            for group in Group.objects.filter(pk__in=pks).distinct():
                group.has_active_roles = self.request.user.role_set.filter(group=group, active=True).exists()
                group.scope_roles = self.request.user.role_set.filter(group=group, scope__isnull=False)
                self.groups.append(group)

    def process_request(self):
        if 'scope' in self.request.GET:
            from djangoplus.admin.models import Group
            self.request.user.role_set.update(active=False)
            self.request.user.permission_mapping = {}
            self.request.user.scope_id = self.request.GET['scope'] or None
            self.request.user.save()

            for group in Group.objects.filter(pk__in=self.request.GET.getlist('groups[]')):
                group.active_scope_roles = []
                for role in self.request.user.role_set.filter(group=group):
                    if role.scope and self.request.user.scope:
                        active = role.scope == self.request.user.scope
                        if not active:
                            organization = role.scope.is_organization()
                            if organization:
                                active = organization.get_units().filter(pk=self.request.user.scope.id).exists()
                        role.active = active
                    else:
                        role.active = True
                    role.save()
            
            self.request.user.check_role_groups()
            # if self.request.user.is_authenticated:
            #     print(self.request.user.permission_mapping.get('Diario:Diario'))
            raise ComponentHasResponseException(HttpResponse())


class QrCode(Component):
    def __init__(self, text, request=None, width=200, height=200):
        super(QrCode, self).__init__(text, request)
        self.text = text
        self.width = width
        self.height = height
        self.base64 = None

    def __str__(self):
        if self.as_pdf:
            qr = qrcode.QRCode()
            qr.add_data(self.text)
            image = qr.make_image()
            file_path = tempfile.mktemp()
            buffer = open(file_path, 'wb')
            image.save(buffer, format="JPEG")
            self.base64 = base64.b64encode(open(file_path, 'rb').read()).decode('utf-8')
        return super(QrCode, self).__str__()


class ProgressBar(Component):
    def __init__(self, percentual, request=None):
        self.percentual = percentual
        super(ProgressBar, self).__init__(None, request)


class Table(Component):
    def __init__(self, request, title, header=None, rows=None, footer=None, enumerable=True, note=None, icon=None):
        super(Table, self).__init__(title, request)
        self.title = title
        self.header = header or []
        self.rows = rows or []
        self.footer = footer or []
        self.enumerable = enumerable
        self.note = note
        self.icon = icon


class ModelTable(Table):
    def __init__(self, request, title, qs, list_display=(), note=None):

        header = []
        rows = []

        if not list_display:
            list_display = get_metadata(qs.model, 'list_display')

        for lookup in list_display:
            column_name = get_fiendly_name(qs.model, lookup, as_tuple=False)
            header.append(column_name)

        for obj in qs:
            row = []
            for lookup in list_display:
                row.append(getattr2(obj, lookup))
            rows.append(row)

        super(ModelTable, self).__init__(request, title, header, rows, note=note)


class QueryStatisticsTable(Table):

    formatter_name = 'statistics'

    def __init__(self, queryset_statistics, request=None, title=None, icon=None):
        if queryset_statistics.groups:
            header = []
            rows = []
            footer = []
            if queryset_statistics.series:
                for i, group in enumerate(queryset_statistics.groups):
                    row = [group]
                    for value in queryset_statistics.series[i]:
                        row.append(value)
                    if len(queryset_statistics.series) > 1:
                        row.append(queryset_statistics.xtotal[i])
                    rows.append(row)
                if len(queryset_statistics.series) > 1:
                    footer = [' '] + queryset_statistics.ytotal + [queryset_statistics.total()]
                    header = [' '] + queryset_statistics.labels + [' ']
                else:
                    header = [' '] + queryset_statistics.labels
        else:
            header = []
            rows = []
            footer = []
            if queryset_statistics.series:
                for i, serie in enumerate(queryset_statistics.series[0]):
                    rows.append([queryset_statistics.labels[i], queryset_statistics.series[0][i]])
                if len(queryset_statistics.series[0]) > 1:
                    footer.append('')
                    footer.append(queryset_statistics.total())
        super(QueryStatisticsTable, self).__init__(request, title, header, rows, footer, enumerable=False)
        self.queryset_statistics = queryset_statistics
        self.symbol = queryset_statistics.symbol


class ModelReport(Component):
    def __init__(self, request, title, qs, list_display=(), list_filter=(), distinct=False):
        super(ModelReport, self).__init__(title, request)
        self.title = title
        self.qs = qs
        self.components = []
        self.filters = []

        if list_filter:
            form = forms.Form(request, method='GET')
            form.icon = 'fa-file-text-o'
            form.title = ''
            form.submit_label = 'Gerar Relatório'
            for field_name in list_filter:
                field = get_field(qs.model, field_name)
                if hasattr(field, 'choices') and field.choices:
                    form.fields[field_name] = forms.ChoiceField(
                        choices=[['', '']]+field.choices,
                        label=field.verbose_name,
                        required=False
                    )
                else:
                    form.fields[field_name] = forms.ModelChoiceField(
                        field.remote_field.model.objects.all(),
                        label=field.verbose_name,
                        required=False
                    )
            if form.is_valid():
                for field_name in list_filter:
                    value = form.cleaned_data[field_name]
                    if value:
                        qs = qs.filter(**{field_name : value})
                        self.filters.append((form.fields[field_name].label, value))
            self.form = form
        if distinct:
            pks = qs.values_list('pk', flat=True).order_by('pk').distinct()
            qs = qs.model.objects.filter(pk__in=pks)

        order_by = get_metadata(qs.model, 'order_by', iterable=True)
        if order_by:
            qs = qs.order_by(*order_by)
        table_description = get_metadata(qs.model, 'verbose_name_plural')
        self.table = ModelTable(request, table_description, qs, list_display)

    def count(self, vertical_key, horizontal_key=None, symbol=None, add_table=True, add_chart=False):
        statistics = self.qs.count(vertical_key, horizontal_key)
        statistics.title = 'Resumo'
        statistics.symbol = symbol
        if add_table:
            self.components.append(statistics.as_table(self.request))
        if add_chart:
            self.components.append(statistics.as_chart(self.request))

class MultiScheduleTable(Component):
    def __init__(self, data=(), request=None, title=None, icon=None, form_prefix=None):
        self.schedule_tables = []
        for shift, shift_data in data:
            self.schedule_tables.append(ScheduleTable(shift_data, request=request, title=shift, form_prefix=form_prefix))


class ScheduleTable(Component):
    WEEK_DAYS = DAY_NAMES

    def __init__(self, data=(), request=None, title=None, icon=None, form_prefix=None):
        super().__init__(title, request)
        self.title = title
        self.icon = icon
        self.rows = []
        self.form_prefix = form_prefix
        self.has_item = False

        if data:
            intervals, scheduled_times = data
            for interval in intervals:
                self.add_interval(str(interval))
            for scheduled_time in scheduled_times:
                self.add(*scheduled_time)

    def add_interval(self, interval):
        self.rows.append((str(interval), [], [], [], [], [], [], []))

    def add(self, week_day, interval, value='X', hint=None):
        week_day_index = None
        for i, wd in enumerate(ScheduleTable.WEEK_DAYS):
            if wd == week_day:
                week_day_index = i + 1
                break
        if week_day_index:
            for row in self.rows:
                if row[0] == str(interval):
                    row[week_day_index].append(value)
                    self.has_item = True
                    break


class ColorBox(Component):

    def __init__(self, color):
        super().__init__()
        self.color = color




# -*- coding: utf-8 -*-
import traceback
from django.apps import apps
from django.conf import settings
from django.shortcuts import render
from django.http import HttpResponse
from django.views.static import serve
from djangoplus.cache import CACHE
from djangoplus.utils import permissions
from django.http import HttpResponseRedirect
from djangoplus.utils.http import return_response
from djangoplus.utils.serialization import json_serialize
from djangoplus.utils.storage import dropbox
from jinja2 import Template
from django.views.defaults import page_not_found
from django.utils.translation import ugettext as _
from django.core.exceptions import ValidationError
from djangoplus.ui.components.forms import factory
from django.views.decorators.csrf import csrf_exempt
from djangoplus.ui.components.panel import ModelDashboard
from django.http.response import HttpResponseForbidden
from djangoplus.ui.components import ComponentHasResponseException
from djangoplus.ui.components.paginator import Paginator
from django.contrib.contenttypes.models import ContentType
from djangoplus.db.models import QuerySet
from djangoplus.ui.components.navigation.breadcrumbs import httprr
from djangoplus.utils.metadata import list_related_objects, is_many_to_many, is_one_to_one, get_metadata, \
    check_condition, is_one_to_many, getattr2, is_one_to_many_reverse, get_parameters_names, count_parameters_names, \
    get_role_value_for_action, get_role_values_for_condition
from django.contrib import auth

def listt(request, app, cls, subset=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')
    title = get_metadata(_model, 'verbose_name_plural')
    subsetp = None
    list_display = None
    list_filter = None
    search_fields = None
    if subset:
        subset_func = getattr(_model.objects.get_queryset(), subset)
        can_view = get_metadata(subset_func, 'can_view')
        list_display = get_metadata(subset_func, 'list_display')
        list_filter = get_metadata(subset_func, 'list_filter')
        search_fields = get_metadata(subset_func, 'search_fields')
        title = '{} - {}'.format(title, get_metadata(subset_func, 'verbose_name'))
    else:
        tid = request.GET.get('tid')
        subsetp = request.GET.get('tab{}'.format(tid))
        if tid and subsetp:
            subset_func = getattr(_model.objects.get_queryset(), subsetp)
            subset_title = get_metadata(subset_func, 'verbose_name')
            can_view = get_metadata(subset_func, 'can_view')
            title = '{} - {}'.format(title, get_metadata(subset_func, 'verbose_name'))
            if not permissions.check_group_or_permission(request, can_view):
                return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))
        else:
            permission = '{}.list_{}'.format(app, cls)
            if not request.user.has_perm(permission):
                return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    qs = _model.objects.all(request.user)
    if subset:
        qs = getattr(qs, subset)()
    list_subsets = subset and [subset] or None

    paginator = Paginator(request, qs, title, list_subsets=list_subsets, is_list_view=True,
                          list_display=list_display, list_filter=list_filter, search_fields=search_fields)
    paginator.process_request()

    paginator.load_actions()
    return render(request, 'default.html', locals())


def add(request, app, cls, pk=None, related_field_name=None, related_pk=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    obj = pk and _model.objects.all(request.user).filter(pk=pk).first() or _model()
    obj.request = request
    obj._user = request.user

    title = pk and str(obj) or get_metadata(_model, 'verbose_name')

    if related_field_name is None:

        if obj.pk:
            if not permissions.has_edit_permission(request, _model) or not permissions.can_edit(request, obj):
                return HttpResponseForbidden()
        else:
            if not permissions.has_add_permission(request, _model) or not permissions.can_add(request, obj):
                return HttpResponseForbidden()

        form = factory.get_register_form(request, obj)
        title = form.title

    elif is_one_to_many(_model, related_field_name):
        if not permissions.can_add(request, obj) and not permissions.can_edit(request, obj):
            return HttpResponseForbidden()
        form = factory.get_one_to_many_form(request, obj, related_field_name)

    elif is_many_to_many(_model, related_field_name):
        if not permissions.can_edit_field(request, obj, related_field_name):
            return HttpResponseForbidden()
        form = factory.get_many_to_many_form(request, obj, related_field_name, related_pk)

    elif is_one_to_many_reverse(_model, related_field_name):
        form = factory.get_many_to_many_reverse_form(request, obj, related_field_name)

    elif is_one_to_one(_model, related_field_name):
        if not permissions.can_add(request, obj) and not permissions.can_edit(request, obj):
            return HttpResponseForbidden()
        form = factory.get_one_to_one_form(request, obj, related_field_name, related_pk)
    else:
        # many to one
        for rel in list_related_objects(_model):
            if hasattr(rel, 'get_accessor_name'):
                if rel.get_accessor_name() in ('{}_set'.format(related_field_name), related_field_name):
                    related_queryset = rel.related_model.objects.all(request.user)
                    related_obj = related_pk and related_queryset.get(pk=related_pk) or rel.related_model()
                    related_obj.request = request
                    setattr(related_obj, rel.field.name, obj)
                    setattr(related_obj, '{}_id'.format(rel.field.name), obj.pk)
                    if related_pk:
                        if not permissions.has_edit_permission(
                                request, rel.related_model) or not permissions.can_edit(request, related_obj):
                            return HttpResponseForbidden()
                    else:
                        if not permissions.has_add_permission(
                                request, rel.related_model) or not permissions.can_add(request, related_obj):
                            return HttpResponseForbidden()
                    form = factory.get_many_to_one_form(request, obj, rel.get_accessor_name(), related_obj)
                    title = form.title

    if form.is_valid():
        is_editing = form.instance.pk is not None
        try:
            form.save()
            obj = form.instance
            if 'select' in request.GET:
                return HttpResponse('{}|{}|{}'.format(obj.pk, obj, request.GET['select']));
            elif related_field_name:
                message = _('Action successfully performed.')
                url = '..'
            else:
                message = get_metadata(form.instance.__class__, 'add_message')
                if message and not is_editing:
                    if hasattr(obj, 'get_absolute_url'):
                        url = obj.get_absolute_url()
                    else:
                        url = '/view/{}/{}/{}/'.format(
                            get_metadata(obj.__class__, 'app_label'), obj.__class__.__name__.lower(), obj.pk)
                else:
                    url = '..'
                if is_editing:
                    message = message or _('Action successfully performed.')
                else:
                    message = message or _('Registration successfully performed.')
            return httprr(request, url, message)
        except ValidationError as e:
            form.add_error(None, str(e.message))
    return render(request, 'default.html', locals())


def view(request, app, cls, pk, tab=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    obj = _model.objects.all(request.user).filter(pk=pk).first()
    obj.request = request
    obj._user = request.user

    if 'one_to_many_count' in request.GET:
        # TODO create a specific view for this purpose
        return HttpResponse(getattr2(obj, request.GET['one_to_many_count']))

    if not permissions.can_view(request, obj):
        return HttpResponseForbidden()

    title = str(obj)
    parent = request.GET.get('parent', None)
    printable = get_metadata(_model, 'pdf', False)
    widget_panel = ModelDashboard(request, obj, tab, parent, printable=printable)
    widget_panel.process_request()

    if widget_panel.model_panel.message:
        return httprr(request, request.get_full_path(), widget_panel.model_panel.message)

    log_data = get_metadata(obj.__class__, 'log', False)
    if log_data and request.user.is_superuser and request.user.has_perm('admin.list_log'):
        url = '/log/{}/{}/'.format(app, cls)
        widget_panel.model_panel.drop_down.add_action('{} {}'.format(_('View'), _('Log')), url, 'ajax', 'fa fa-history')

    return render(request, 'default.html', locals())


def action(request, app, cls, action_name, pk=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    for group in CACHE['INSTANCE_ACTIONS'][_model]:
        if action_name in CACHE['INSTANCE_ACTIONS'][_model][group]:
            break

    form_action = CACHE['INSTANCE_ACTIONS'][_model][group][action_name]
    action_verbose_name = form_action['verbose_name']
    action_can_execute = form_action['can_execute']
    action_condition = form_action['condition']
    action_function = form_action['function']
    action_message = 'message' in form_action and form_action['message'] or None
    action_permission = '{}.{}'.format(_model._meta.app_label, action_function.__name__)
    action_input = form_action['input']
    action_display = form_action['display']
    action_style = form_action['style']
    action_redirect = form_action['redirect_to']

    obj = pk and _model.objects.all(request.user).distinct().get(pk=pk) or _model()
    obj.request = request
    obj._user = request.user
    title = action_verbose_name
    redirect_to = None

    if check_condition(request.user, action_condition, obj) and (
            not action_can_execute or permissions.check_group_or_permission(request, action_permission)):
        f_return = None
        func = getattr(_model, action_function.__name__, action_function)
        form = factory.get_action_form(request, obj, form_action)

        if form.fields and form.is_valid() or not form.fields:
            if 'instance' in form.fields:
                obj = form.cleaned_data['instance']
            func = getattr(obj, action_function.__name__, action_function)
            params = []
            for param in get_parameters_names(func, include_annotated=True):
                if param in form.cleaned_data:
                    params.append(form.cleaned_data[param])
                else:
                    params.append(get_role_value_for_action(func, request.user, param))
            try:
                f_return = func(*params)
                if not action_redirect:
                    if count_parameters_names(func) > 0 or action_display:
                        redirect_to = '..'
                    else:
                        redirect_to = '.'
                else:
                    redirect_to = Template(action_redirect).render(obj=obj)
            except ValidationError as e:
                if form.fields:
                    form.add_error(None, e.message)
                    redirect_to = None
                else:
                    return httprr(request, '.', e.message, error=True)

        if f_return and type(f_return) != str:
            template_name = '{}.html'.format(action_function.__name__)
            return return_response(f_return, request, title, action_style, template_name)

        elif redirect_to:
            return httprr(request, redirect_to, f_return or action_message)

        if form.title == _('Form'):
            form.title = action_verbose_name
        if form.submit_label == _('Send'):
            form.submit_label = action_verbose_name
        return render(request, 'default.html', locals())
    else:
        return HttpResponseForbidden()


def delete(request, app, cls, pk, related_field_name=None, related_pk=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    obj = _model.objects.all(request.user).get(pk=pk)
    obj._request = request
    obj._user = request.user

    if permissions.can_delete(request, obj):
        if related_field_name:
            getattr(obj, related_field_name).remove(related_pk)
            return httprr(request, '..', _('Deletion successfully performed.'))
        else:
            title = '{} {}'.format(_('Delete'), str(obj))
            form = factory.get_delete_form(request, obj)
            if form.is_valid():
                obj.delete()
                return httprr(request, '..', _('Action successfully performed.'))
            return render(request, 'delete.html', locals())
    else:
        return HttpResponseForbidden()


def log(request, app, cls, pk=None):

    if not request.user.is_authenticated:
        return httprr(request, '/admin/login/?next={}'.format(request.get_full_path()))

    try:
        _model = apps.get_model(app, cls)
    except LookupError as e:
        return page_not_found(request, e, 'error404.html')

    if pk:
        obj = _model.objects.get(pk=pk)
        qs = obj.get_logs()
        title = 'Log - {}'.format(obj)
    else:
        content_type = ContentType.objects.get_for_model(_model)
        qs = content_type.log_set.all()
        title = 'Logs - {}'.format(get_metadata(_model, 'verbose_name_plural'))

    paginator = Paginator(request, qs, 'Log')
    return render(request, 'default.html', locals())


def dispatcher(request, app, view_name, params):

    params = params.split('/')[0:-1]

    full_app_name = settings.APP_MAPPING.get(app, app)
    fromlist = full_app_name.split('.')

    try:
        views = __import__('{}.views'.format(full_app_name), fromlist=list(map(str, fromlist)))
        func = getattr(views, view_name)
    except ComponentHasResponseException as e:
        raise e
    except (ImportError, TypeError, AttributeError) as e:
        traceback.print_exc()
        return page_not_found(request, e, 'error404.html')

    try:
        return func(request, *params)
    except ComponentHasResponseException as e:
        raise e
    except Exception as e:
        print(e)
        traceback.print_exc()
        #return server_error(request, 'error500.html')
        raise e


def cloud(request, path, document_root=None, show_indexes=False):
    storage = dropbox.DropboxStorage()
    if storage.exists_locally(path):
        return serve(request, path, document_root=document_root, show_indexes=show_indexes)
    else:
        return HttpResponseRedirect(storage.remote_url('/{}'.format(path)))


@csrf_exempt
def api(request, app_label, model_name, arg1=None, arg2=None):
    from djangoplus.admin.models import User

    obj = None
    form = None
    result = None
    errors = None
    func_name = None
    message = None
    args = {}

    http_authorization = request.META.get('HTTP_AUTHORIZATION')
    if http_authorization and 'Token' in http_authorization:
            page= int(request.GET.get('page', '') or 0)
            token = http_authorization[5:].strip()
            user = User.objects.get(token=token)
            request.user = user
            if app_label and model_name:
                model = apps.get_model(app_label, model_name)
                if model == User and str(user.pk) == arg1:
                    data = dict(name=user.name, username=user.username, photo=user.photo.url, email=user.email)
                    return HttpResponse(
                        json_serialize(result, request.GET, errors=[], message=_('Action successfully performed.'), exception=None, result=data)
                    )
                # model @action or @meta | manager @subset [@action or @meta]
                elif arg1 and arg2:
                    # model
                    if arg1.isdigit():
                        obj = model.objects.all(user).get(pk=arg1)
                        if arg2:
                            # @action
                            group = None
                            for group in CACHE['INSTANCE_ACTIONS'][model]:
                                if arg2 in CACHE['INSTANCE_ACTIONS'][model][group]:
                                    action_dict = CACHE['INSTANCE_ACTIONS'][model][group][arg2]
                                    message = action_dict.get('message')
                                    func_name = arg2
                                    break
                            if func_name:
                                func = getattr(obj, func_name)
                                form_action = CACHE['INSTANCE_ACTIONS'][model][group][arg2]
                                # with input
                                if count_parameters_names(func) > 0:
                                    form = factory.get_action_form(request, obj, form_action)
                            # it is not @action
                            else:
                                # @meta
                                for method in CACHE['INSTANCE_METHODS'].get(model, []):
                                    if method['function'] == arg2:
                                        func_name = arg2
                                        func = getattr(obj, func_name)
                                        break
                    # manager @subset
                    else:
                        obj = model.objects.all(user)
                        pks = request.GET.get('ids', [])
                        if pks:
                            obj = obj.filter(pk__in=pks)
                        group = None
                        # @action
                        for group in CACHE['QUERYSET_ACTIONS'][model]:
                            if arg2 in CACHE['QUERYSET_ACTIONS'][model][group]:
                                action_dict = CACHE['QUERYSET_ACTIONS'][model][group][arg2]
                                message = action_dict.get('message')
                                func_name = arg2
                                break
                        if func_name:
                            func = getattr(obj, func_name)
                            form_action = CACHE['QUERYSET_ACTIONS'][model][group][arg2]
                            # with input
                            if count_parameters_names(func) > 0:
                                form = factory.get_class_action_form(request, obj.model, form_action, func)
                        # it is not @action
                        else:
                            # @meta
                            for method in CACHE['MANAGER_METHODS'].get(model, []):
                                if method['function'] == arg1:
                                    func_name = arg2
                                    func = getattr(obj, func_name)
                                    break
                # model | manager @subset or @action or @meta
                elif arg1:
                    if arg1.isdigit():
                        # model
                        obj = model.objects.all(user).get(pk=arg1)
                        if request.method.lower() == 'get':
                            result = obj
                        elif request.method.lower() == 'delete':
                            result = None
                            obj.delete()
                            message = _('Deletion successfully performed.')
                        else:
                            form = factory.get_register_form(request, obj)
                    else:
                        # manager @subset or @action or @meta
                        obj = model.objects.all(user)
                        pks = request.GET.get('ids', [])
                        if pks:
                            obj = obj.filter(pk__in=pks)
                        group = None

                        # @action
                        func = None
                        action_dict = None
                        for group in CACHE['QUERYSET_ACTIONS'][model]:
                            if arg1 in CACHE['QUERYSET_ACTIONS'][model][group]:
                                action_dict = CACHE['QUERYSET_ACTIONS'][model][group][arg1]
                                message = action_dict.get('message')
                                func = getattr(obj, arg1)
                                func_name = arg1
                                break
                        if not func_name:
                            for group in CACHE['CLASS_ACTIONS'][model]:
                                if arg1 in CACHE['CLASS_ACTIONS'][model][group]:
                                    action_dict = CACHE['CLASS_ACTIONS'][model][group][arg1]
                                    message = action_dict.get('message')
                                    func = getattr(obj, arg1)
                                    func_name = arg1
                                    break
                        if func_name:
                            # with input
                            if count_parameters_names(func) > 0:
                                form = factory.get_class_action_form(request, obj.model, action_dict, func)
                        # it is not @action
                        else:
                            # @subset or @meta
                            for subset in CACHE['SUBSETS'][model]:
                                if arg1 == subset['name']:
                                    func_name = arg1
                                    break
                            if func_name:
                                func = getattr(obj, func_name)
                            # it is not @subset
                            else:
                                # @meta
                                for method in CACHE['MANAGER_METHODS'].get(model, []):
                                    if method['function'] == arg1:
                                        func_name = arg1
                                        func = getattr(obj, func_name)
                                        break
                # list or save
                else:
                    if request.method.lower() == 'get':
                        obj = model.objects.all(user)
                        func_name = 'all'
                    else:
                        form = factory.get_register_form(request, model())

            # list or action
            if func_name:
                if form:
                    if form.is_valid():
                        params = []
                        func = getattr(obj, func_name)
                        for param in get_parameters_names(func, include_annotated=True):
                            if param in form.cleaned_data:
                                params.append(form.cleaned_data[param])
                            else:
                                params.append(get_role_value_for_action(func, request.user, param))
                        try:
                            result = func(*params)
                            if hasattr(result, 'all'):
                                result = result[page*10:(page*10)+10]
                            
                        except ValidationError as e:
                            message = None
                            form.add_error(None, e.message)
                    else:
                        errors = form.errors
                elif obj:
                    try:
                        func = getattr(obj, func_name)
                        args = get_role_values_for_condition(func, request.user)
                        result = func(*args)
                        if hasattr(result, 'all'):
                            result = result[page*10:(page*10)+10]
                    except ValidationError as e:
                        errors = {'__all__': e.message}
                        message = None
                return HttpResponse(
                    json_serialize(result, request.GET, errors=errors, message=message, exception=None)
                )
            # add or edit
            elif form:
                if form.is_valid():
                    result = form.save()
                    message = _('Action successfully performed.')
                else:
                    errors = form.errors
                return HttpResponse(
                    json_serialize(result, request.GET, errors=errors, message=message, exception=None)
                )
            # get or delete
            else:
                return HttpResponse(
                    json_serialize(result, request.GET, errors=errors, message=message, exception=None)
                )
    else:
        user = auth.authenticate(username=request.POST['username'], password=request.POST['password'])
        if user is not None:
            return HttpResponse(
                json_serialize(result, request.GET, errors=[], message=_('Action successfully performed.'), exception=None, result=dict(token=user.token, id=user.id))
            )
        else:
            return HttpResponseForbidden()



# -*- coding: utf-8 -*-

import sys
import datetime
from django.core import signing
from django.contrib import auth
from django.conf import settings
from djangoplus.cache import CACHE
from djangoplus.ui.components import forms
from django.utils.translation import ugettext as _
from django.contrib.auth.hashers import make_password
from djangoplus.ui.components.navigation.breadcrumbs import httprr
from djangoplus.admin.models import Group, User, Unit, Settings, Organization


class LoginForm(forms.Form):
    
    ORGANIZATION = 'organization'
    UNIT = 'unit'
    
    username = forms.CharField(label=_('User'))
    password = forms.PasswordField(label=_('Password'))

    def __init__(self, request, scope=None, organization=None, unit=None, *args, **kwargs):
        super(LoginForm, self).__init__(request, *args, **kwargs)
        self.scope = None
        self.fields['username'].widget.attrs['class'] = 'input-sm bounceIn animation-delay2'
        if scope:
            if scope == LoginForm.ORGANIZATION or scope == (
                    CACHE['ORGANIZATION_MODEL'] and CACHE['ORGANIZATION_MODEL'].__name__.lower()):
                if organization:
                    self.scope_display = organization
                else:
                    organizations = Organization.objects.all()
                    self.fields['login_scope'] = forms.ModelChoiceField(organizations)
                self.scope = LoginForm.ORGANIZATION
            elif scope == LoginForm.UNIT or scope == (CACHE['UNIT_MODEL'] and CACHE['UNIT_MODEL'].__name__.lower()):
                if unit:
                    self.scope_display = unit
                else:
                    if organization:
                        self.scope_display = organization
                        units = organization.get_units()
                    else:
                        units = Unit.objects.all()
                    self.fields['login_scope'] = forms.ModelChoiceField(units)
                self.scope = LoginForm.UNIT
        self.organization = organization
        self.unit = unit

    def clean(self):
        cleaned_data = super(LoginForm, self).clean()
        username = cleaned_data.get('username')
        password = cleaned_data.get('password')
        if not User.objects.filter(username=username).exists():
            raise forms.ValidationError(_('User not registered.'))
        user = auth.authenticate(username=username, password=password)
        if user is not None:
            if user.active:
                user.permission_mapping = ''
                user.organization = None
                user.unit = None
                if self.scope == LoginForm.ORGANIZATION:
                    user.organization = cleaned_data.get('login_scope', self.organization)
                    is_organization_user = user.role_set.filter(scope__in=(user.organization, 0)).exists()
                    if not is_organization_user:
                        raise forms.ValidationError('{} {} {}'.format(username, _('is not user of'), user.organization))
                elif self.scope == LoginForm.UNIT:
                    user.unit = cleaned_data.get('login_scope', self.unit)
                    is_unit_user = user.role_set.filter(scope__in=(user.unit, 0)).exists()
                    is_organization_user = CACHE['ORGANIZATION_MODEL'] and user.role_set.filter(
                        scope__in=(user.unit.get_organization(), 0)).exists()
                    if not is_unit_user and not is_organization_user:
                        raise forms.ValidationError('{} {} {}'.format(username, _('is not user of'), user.unit))
                else:
                    roles = user.role_set.all()
                    if roles.count() == 1:
                        role = roles.first()
                        if role.scope:
                            user.scope = role.scope
                user.save()
                auth.login(self.request, user)
                return cleaned_data
            else:
                raise forms.ValidationError(_('User is inactive.'))
        else:
            raise forms.ValidationError(_('Password is incorrect.'))

    def submit(self):
        url = self.request.GET.get('next', '/admin/')
        return httprr(self.request, url, _('You have successfully logged in.'))


class GroupForm(forms.ModelForm):
    name = forms.CharField(label='Name')

    class Meta:
        title = '{} {}'.format(_('Register'), _('Group'))
        submit_label = _('Save')
        icon = 'fa-users'
        model = Group
        fields = ('name',)


class UserForm(forms.ModelForm):
    is_superuser = forms.BooleanField(label=_('Superuser'), required=False)
    new_password = forms.PasswordField(label=_('Password'), required=False)

    class Meta:
        model = User
        fields = ('username', 'name', 'active', 'email', 'is_superuser', 'photo')
        title = '{} {}'.format(_('Register'), _('User'))
        submit_label = _('Save')
        icon = 'fa-user'

    def __init__(self, *args, **kwargs):
        super(UserForm, self).__init__(*args, **kwargs)
        if not args[0].user.is_superuser:
            del (self.fields['is_superuser'])
            self.fieldsets = (
                (_('Identification'), {'fields': (('name', 'email',), 'photo')}),
                (_('Access'), {'fields': (('username', 'new_password'), ('active',))}),
            )
        else:
            self.fieldsets = (
                (_('Identification'), {'fields': (('name', 'email',), 'photo')}),
                (_('Access'), {'fields': (('username', 'new_password'), ('active', 'is_superuser'))}),
            )

    def save(self, *args, **kwargs):
        user = super(UserForm, self).save(commit=False)
        if 'new_password' in self.cleaned_data and self.cleaned_data['new_password']:
            user.set_password(self.cleaned_data['new_password'])
        user.save()


class RegisterForm(forms.Form):
    name = forms.CharField(label=_('Name'), required=True)
    email = forms.EmailField(label=_('E-mail'), required=True)
    new_password = forms.PasswordField(label=_('Password'), required=True)
    confirm_password = forms.PasswordField(label=_('Password'), required=True, help_text=_('Repeat Password'))

    class Meta:
        title = 'Novo Usuário'
        icon = 'fa-user'
        submit_label = 'Cadastrar'
        captcha = True

    fieldsets = (
        (_('User Data'), {'fields': ('name', 'email')}),
        (_('Access Password'), {'fields': (('new_password', 'confirm_password'),)}),
    )

    def clean(self):
        if not self.data['new_password'] == self.data['confirm_password']:
            raise forms.ValidationError(_('The passwords must be the same.'))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        create = True
        user = User()
        password = self.cleaned_data['new_password']
        user.name = self.cleaned_data['name']
        user.email = self.cleaned_data['email']
        user.username = self.cleaned_data['email']
        user.active = True
        user.is_superuser = False
        user.last_login = None
        user.date_joined = datetime.datetime.now()
        user.username = user.email
        user.set_password(password)
        if create:
            user.save()
            return user
        else:
            token = signing.dumps('{};{};{}'.format(user.name, user.email, password))
            url = '{}/admin/create_user/{}/'.format(settings.SERVER_ADDRESS, token)
            msg = '{}: {}'.format(_('Click on the link to activate your account'), url), settings.SYSTEM_EMAIL_ADDRESS
            user.email_user(_('Account Activation'), msg)
            return None


class ChangePasswordForm(forms.ModelForm):
    new_password = forms.PasswordField(label=_('Password'), required=True)
    confirm_password = forms.PasswordField(label=_('Confirm Password'), required=True, help_text=_('Repeat the password'))

    fieldsets = ((_('Password'), {'fields': ('new_password', 'confirm_password',)}),)

    class Meta:
        model = User
        fields = ('id',)
        title = _('Change Password')
        submit_label = _('Save')
        icon = 'fa-key'

    def clean(self):
        if not self.data.get('new_password') == self.data.get('confirm_password'):
            raise forms.ValidationError(_('The passwords must be the same.'))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        super(ChangePasswordForm, self).save(*args, **kwargs)
        User.objects.filter(pk=self.instance.pk).update(password=make_password(self.cleaned_data['new_password']))
        user = auth.authenticate(username=self.instance.username, password=self.cleaned_data['new_password'])
        auth.login(self.request, user)


class RecoverPassowordForm(forms.Form):
    email = forms.EmailField(label='E-mail')

    class Meta:
        captcha = 'test' not in sys.argv

    def clean(self):
        cleaned_data = super(RecoverPassowordForm, self).clean()
        qs = User.objects.filter(
            email=cleaned_data['email']
        )
        if qs.exists():
            qs.first().send_reset_password_notification()
            return cleaned_data
        else:
            raise forms.ValidationError(_('User not found.'))


class ProfileForm(forms.ModelForm):
    new_password = forms.PasswordField(label=_('Password'), required=False)
    confirm_password = forms.PasswordField(label=_('Confirm Password'), required=False, help_text=_('Repeat the password'))

    class Meta:
        model = User
        fields = ('name', 'photo', 'username', 'email')
        title = _('Edit Profile')
        submit_label = _('Save')
        icon = 'fa-user'

    fieldsets = (
        (_('User Data'), {'fields': (('name', 'username'), 'email', 'photo')}),
        (_('Access Password'), {'fields': (('new_password', 'confirm_password'),)}),
    )

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)
        self.username = self.instance.username
        if not settings.USERNAME_CHANGE:
            self.fields['username'].widget.attrs.update(readonly='readonly')

    def clean_confirm_password(self):
        if not self.data.get('new_password') == self.data.get('confirm_password'):
            raise forms.ValidationError(_('The passwords must be the same.'))
        return self.cleaned_data

    def save(self, *args, **kwargs):
        super(ProfileForm, self).save(*args, **kwargs)
        password = self.cleaned_data['new_password']

        if password:
            self.instance.set_password(password)
            self.instance.save()
            user = auth.authenticate(username=self.cleaned_data.get('username'), password=password)
        else:
            user = self.instance
        user.backend = self.request.session[auth.BACKEND_SESSION_KEY]
        auth.login(self.request, user)

        if self.username != user.username:
            for model in CACHE['ROLE_MODELS']:
                attr = CACHE['ROLE_MODELS'][model]['username_field']
                model.objects.filter(**{attr: self.username}).update(**{attr: self.request.user.username})


class SettingsForm(forms.ModelForm):
    class Meta:
        title = _('Settings')
        model = Settings
        exclude = ()

    def __init__(self, *args, **kwargs):
        kwargs['instance'] = Settings.default()
        super(SettingsForm, self).__init__(*args, **kwargs)



# Generated by Django 2.0.3 on 2018-04-16 18:54

from django.db import migrations


def migrate(apps, schema_editor):
    Organization = apps.get_model('admin', 'Organization')
    Unit = apps.get_model('admin', 'Unit')

    if Organization.__subclasses__() or Unit.__subclasses__():
        print('\nExecute the following command if you are using sqlite3')
        print('echo .dump | sqlite3 sqlite.db > x.sql && sed \'s/"scope_ptr"/"scope_ptr_id"/g\' x.sql > y.sql && rm x.sql sqlite.db && sqlite3 sqlite.db < y.sql && rm y.sql\n')


class Migration(migrations.Migration):

    dependencies = [
        ('admin', '0006_auto_20180416_1058'),
    ]

    operations = [
        migrations.RunPython(migrate)
    ]



# -*- coding: utf-8 -*-
import re
import platform
import os
import signal
from time import sleep
from subprocess import Popen, PIPE, DEVNULL
from django.conf import settings

from djangoplus.utils import to_ascii


class Subtitle(object):

    @staticmethod
    def display(message, duration=4):
        import tkinter as tk
        root = tk.Tk()
        root.overrideredirect(True)
        root.wm_attributes("-topmost", True)
        root.attributes('-alpha', 0.8)
        root.configure(background='black')
        lines = list()
        break_line = False
        for i, letter in enumerate(message):
            if break_line or i and i % 50 == 0:
                if letter == ' ':
                    lines.append('\n')
                    break_line = False
                else:
                    break_line = True
            lines.append(letter)
        message = ''.join(lines)
        line_breaks = message.count('\n')
        if 'darwin' in platform.system().lower():
            font_size, width, top, right = 30, 82, int(root.winfo_screenwidth() / 2 - 20 * 35), int(
                root.winfo_screenheight() - [90, 120, 156][line_breaks])
        else:
            font_size, width, top, right = 20, 70, int(root.winfo_screenwidth() / 2 - 70 * 7.5), int(
                root.winfo_screenheight() - [77, 110, 140][line_breaks])
        label = tk.Label(root, text=message, font=("Helvetica", font_size), width=width, height=2 + line_breaks)
        label.configure(foreground="white", background='black')
        label.pack(expand=tk.YES, fill=tk.BOTH)
        root.geometry("+{}+{}".format(top, right))
        root.after(1000 * duration, lambda: root.destroy())
        root.mainloop()


class VideoRecorder(object):
    def __init__(self):
        self.proccess = None

    def start(self):
        if 'darwin' in platform.system().lower():
            list_divices_procces = Popen('ffmpeg -f avfoundation -list_devices true -i ""'.split(), stdout=PIPE, stderr=PIPE)
            output, err = list_divices_procces.communicate()
            i = 0
            for line in err.decode('utf-8').split('\n'):
                if 'capture screen' in line.lower():
                    for i in range(0, 3):
                        if '[{}]'.format(i) in line:
                            break
            self.proccess = Popen(['ffmpeg', '-y', '-f', 'avfoundation', '-i', str(i), '/tmp/video.mkv'], stdin=DEVNULL, stdout=DEVNULL, stderr=DEVNULL)
        else:
            xrandr = Popen("xrandr".split(), stdout=PIPE)
            sizes = xrandr.stdout.readlines()
            size = re.findall(r'\d+x\d+', [line.decode('utf-8').strip().split()[0] for line in sizes if '*' in line.decode('utf-8')][0])[0]
            cmd = 'ffmpeg -video_size {} -framerate 25 -f x11grab -i :0.0+0,0 -f pulse -ac 2 -i default /tmp/video.mkv'
            self.proccess = Popen(cmd.format(size).split(), stdin=DEVNULL, stdout=DEVNULL, stderr=DEVNULL)
        sleep(5)

    def stop(self, title=None, output_dir=None, audio_file_path=None):

        if self.proccess:
            os.kill(self.proccess.pid, signal.SIGTERM)
            if title:
                tmp_file_path = '/tmp/video.mkv'
                if not output_dir:
                    output_dir = os.path.join(os.path.join(os.path.expanduser('~')), 'Desktop')
                    if not os.path.exists(output_dir):
                        output_dir = os.path.join(os.path.expanduser('~'))
                if not os.path.exists(output_dir):
                    os.mkdir(output_dir)

                output_file_path = '{}/{}.mkv'.format(output_dir, to_ascii(title).replace(' ', '_'))
                if os.path.exists(output_file_path):
                    os.unlink(output_file_path)

                if self.proccess:
                    os.kill(self.proccess.pid, signal.SIGTERM)
                    sleep(2)
                    os.rename(tmp_file_path, output_file_path)

                    print('Video Ouput: {}'.format(output_file_path))
                    if audio_file_path and os.path.exists(audio_file_path) and 'darwin' in platform.system().lower():
                        output_audio_path = '/tmp/video-audio.mkv'
                        combine_audio_cmd = 'ffmpeg -y -i {} -i {} -c copy -map 0:0 -map 1:0 -shortest {}  > /dev/null 2>&1'
                        os.system(combine_audio_cmd.format(output_file_path, audio_file_path, output_audio_path))
                        sleep(5)
                        os.rename(output_audio_path, output_file_path)

                    cmd = 'ffmpeg -y -i {} -metadata title="{}" {}'.format(output_file_path, title, tmp_file_path)
                    Popen(cmd, stdin=DEVNULL, stdout=DEVNULL, stderr=DEVNULL, shell=True).wait()
                    os.rename(tmp_file_path, output_file_path)

                return output_file_path
        return None

    @staticmethod
    def metadata(file_path):
        cmd = 'ffmpeg -y -i {} -f ffmetadata /tmp/metadata.txt'.format(file_path)
        Popen(cmd, stdin=DEVNULL, stdout=DEVNULL, stderr=DEVNULL, shell=True).wait()
        metadata = {}
        for line in open('/tmp/metadata.txt').readlines():
            if '=' in line:
                tokens = line.strip().split('=')
                metadata[tokens[0]] = tokens[1]
        return metadata


class VideoUploader(object):

    SCOPES = [
        'https://www.googleapis.com/auth/youtube',
        'https://www.googleapis.com/auth/youtube.force-ssl',
        'https://www.googleapis.com/auth/youtube.readonly',
        'https://www.googleapis.com/auth/youtube.upload'
    ]
    API_SERVICE_NAME = 'youtube'
    API_VERSION = 'v3'

    AUTH_URI = 'https://accounts.google.com/o/oauth2/auth'
    TOKEN_URI = 'https://www.googleapis.com/oauth2/v3/token'
    AUTH_CERT_URL = 'https://www.googleapis.com/oauth2/v1/certs'

    PROJECT_ID = 'youtube-215812'
    CLIENT_ID = '289626547199-v2476ui09vqqjgbl607n2lcsq7arq0u0.apps.googleusercontent.com'
    CLIENT_SECRET = 'QjWNSUt_ILAytbnROoSAdv7m'

    def __init__(self):

        from google.oauth2.credentials import Credentials
        from googleapiclient.discovery import build
        from google_auth_oauthlib.flow import InstalledAppFlow

        client_config = {"installed": {
            'client_id': VideoUploader.CLIENT_ID,
            'project_id': VideoUploader.PROJECT_ID, 'auth_uri': VideoUploader.AUTH_URI,
            'token_uri': VideoUploader.TOKEN_URI,
            'auth_provider_x509_cert_url': VideoUploader.AUTH_CERT_URL,
            'client_secret': VideoUploader.CLIENT_SECRET}
        }

        credentials = Credentials(None, refresh_token="1/8nCdTJu1q8xGwPMRiorcWP8XgoOBkwyinSX3MYQRsmo", token_uri="https://accounts.google.com/o/oauth2/token", client_id=VideoUploader.CLIENT_ID, client_secret=VideoUploader.CLIENT_SECRET)

        if not credentials:
            flow = InstalledAppFlow.from_client_config(client_config, VideoUploader.SCOPES)
            credentials = flow.run_console()
            import pdb; pdb.set_trace()

        self.service = build(VideoUploader.API_SERVICE_NAME, VideoUploader.API_VERSION, credentials=credentials)

    def list_videos(self):
        videos_ids = [item['id']['videoId'] for item in self.service.search().list(forMine=True, type="video", part="id,snippet", fields='items(id)').execute()['items']]
        videos = [video for video in self.service.videos().list(part="id,snippet", id=','.join(videos_ids), fields='items(id,snippet/tags,snippet/title,snippet/description)').execute()['items'] if settings.PROJECT_NAME in video.get('snippet', {}).get('tags', [])]
        return videos

    def delete_video(self, video_id):
        self.service.videos().delete(id=video_id).execute()
        print('Video "{}" deleted from Youtube.'.format(video_id))

    def upload_video(self, file_path, title, force=False):
        self.upload_videos([(file_path, title)], force=force)

    def upload_videos(self, tuples, force=False):
        from googleapiclient.http import MediaFileUpload
        uploaded_titles = {}
        for video in self.list_videos():
            uploaded_titles[video['snippet']['title']] = video['id']
        tags = [settings.PROJECT_NAME]
        for file_path, title in tuples:
            if title not in uploaded_titles or force:
                response = None

                if force and title in uploaded_titles:
                    video_id = uploaded_titles[title]
                    self.delete_video(video_id)

                body = dict(
                    snippet=dict(title=title, description='', tags=tags, categoryId='22'),
                    status=dict(privacyStatus='public')
                )

                media_body = MediaFileUpload(file_path, chunksize=-1, resumable=True)
                request = self.service.videos().insert(
                    part=','.join(body.keys()), body=body, media_body=media_body
                )

                while response is None:
                    print('Uploading file "{}"...'.format(file_path))
                    status, response = request.next_chunk()
                    if response is not None:
                        if 'id' in response:
                            url = 'http://www.youtube.com/embed/{}?rel=0&autoplay=1'.format(response['id'])
                            print('The video was successfully uploaded. {}'.format(url))
                        else:
                            exit('The upload failed with an unexpected response: %s' % response)
            else:
                print('The video "{}" has already been uploaded. Force upload if it should be replaced.'.format(title))